/*
 * Implementacin del comando EDIT EXTENDED para la librera HMG.
 * (c) Cristbal Moll [cemese@terra.es]

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this software; see the file COPYING. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text
 contained in this file.

 The exception is that, if you link this code with other
 files to produce an executable, this does not by itself cause the resulting
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking
 this code into it.

 *
 * - Descripcin -
 * ===============
 *      EDIT EXTENDED, es un comando que permite realizar el mantenimiento de una bdd. En principio
 *      est diseado para administrar bases de datos que usen los divers DBFNTX y DBFCDX,
 *      presentando otras bases de datos con diferentes drivers resultados inesperados.
 *
 * - Sintxis -
 * ============
 *      Todos los parmetros del comando EDIT son opcionales.
 *
 *      EDIT EXTENDED                           ;
 *       [ WORKAREA cWorkArea ]                 ;
 *       [ TITLE cTitle ]                       ;
 *       [ FIELDNAMES acFieldNames ]            ;
 *       [ FIELDMESSAGES acFieldMessages ]      ;
 *       [ FIELDENABLED alFieldEnabled ]        ;
 *       [ TABLEVIEW alTableView ]              ;
 *       [ OPTIONS aOptions ]                   ;
 *       [ ON SAVE bSave ]                      ;
 *       [ ON FIND bFind ]                      ;
 *       [ ON PRINT bPrint ]
 *
 *      Si no se pasa ningn parmetro, el comando EDIT toma como bdd de trabajo la del
 *      area de trabajo actual.
 *
 *      [cWorkArea]             Cadena de texto con el nombre del alias de la base de datos
 *                              a editar. Por defecto el alias de la base de datos activa.
 *      [cTitle]                Cadena de texto con el ttulo de la ventana de visualizacin
 *                              de registros. Por defecto se toma el alias de la base de datos
 *                              activa.
 *      [acFieldNames]          Matriz de cadenas de texto con el nombre descriptivo de los
 *                              campos de la base de datos. Tiene que tener el mismo nmero
 *                              de elementos que campos la bdd. Por defecto se toma el nombre
 *                              de los campos de la estructura de la bdd.
 *      [acFieldMessages]       Matriz de cadenas de texto con el texto que aparaecer en la
 *                              barra de estado cuando se este aadiento o editando un registro.
 *                              Tiene que tener el mismo numero de elementos que campos la bdd.
 *                              Por defecto se rellena con valores internos.
 *      [alFieldEnabled]        Matriz de valores lgicos que indica si el campo referenciado
 *                              por la matriz esta activo durante la edicin de registro. Tiene
 *                              que tener el mismo numero de elementos que campos la bdd. Por
 *                              defecto toma todos los valores como verdaderos ( .t. ).
 *      [aTableView]            Matriz de valores lgicos que indica si el campo referenciado
 *                              por la matriz es visible en la tabla. Tiene que tener el mismo
 *                              numero de elementos que campos la bdd. Por defecto toma todos
 *                              los valores como verdaderos ( .t. ).
 *      [aOptions]              Matriz de 2 niveles. el primero de tipo texto es la descripcin
 *                              de la opcin. El segundo de tipo bloque de cdigo es la opcin
 *                              a ejecutar cuando se selecciona. Si no se pasa esta variable,
 *                              se desactiva la lista desplegable de opciones.
 *      [bSave]                 Bloque de codigo con el formato {|aValores, lNuevo| Accion } que
 *                              se ejecutar al pulsar la tecla de guardar registro. Se pasan
 *                              las variables aValores con el contenido del registro a guardar y
 *                              lNuevo que indica si se est aadiendo (.t.) o editando (.f.).
 *                              Esta variable ha de devolver .t. para salir del modo de edicin.
 *                              Por defecto se graba con el cdigo de la funcin.
 *      [bFind]                 Bloque de codigo a ejecutar cuando se pulsa la tecla de busqueda.
 *                              Por defecto se usa el cdigo de la funcin.
 *      [bPrint]                Bloque de cdigo a ejecutar cuando se pulsa la tecla de listado.
 *                              Por defecto se usa el codigo de la funcin.
 *
 *      Ver DEMO.PRG para ejemplo de llamada al comando.
 *
 *
 * - Historial -
 * =============
 *      Mar 03  - Definicin de la funcin.
 *              - Pruebas.
 *              - Soporte para lenguaje en ingls.
 *              - Corregido bug al borrar en bdds con CDX.
 *              - Mejora del control de parmetros.
 *              - Mejorada la funcin de de busqueda.
 *              - Soprte para multilenguaje.
 *              - Versin 1.0 lista.
 *      Abr 03  - Corregido bug en la funcin de busqueda (Nombre del botn).
 *              - Aadido soporte para idioma Ruso (Grigory Filiatov).
 *              - Aadido soporte para idioma Cataln (Por corregir).
 *              - Aadido soporte para idioma Portugus (Clovis Nogueira Jr).
 *              - Aadido soporte para idioma Polaco (Janusz Poura).
 *              - Aadido soporte para idioma Francs (C. Jouniauxdiv).
 *      May 03  - Aadido soporte para idioma Italiano (Lupano Piero).
 *              - Aadido soporte para idioma Alemn (Janusz Poura).
 *              - Cambio del formato de llamada al comando.
 *              - La creacin de ventanas se realiza en funcin del alto y ancho
 *                de la pantalla.
 *              - Se elimina la restriccin de tamao en los nombre de etiquetas.
 *              - Se elimina la restriccin de numero de campos del area de la bdd.
 *              - Se elimina la restriccin de que los campos tipo MEMO tienen que ir
 *                al final de la base de datos.
 *              - Se aade la opcin de visualizar comentarios en la barra de estado.
 *              - Se aade opcin de control de visualizacin de campos en el browse.
 *              - Se modifica el parmetro nombre del area de la bdd que pasa a ser
 *                opcional.
 *              - Se aade la opcin de saltar al siguiente foco mediante la pulsacin
 *                de la tecla ENTER (Solo a controles tipo TEXTBOX).
 *              - Se aade la opcin de cambio del indice activo.
 *              - Mejora de la rutina de busqueda.
 *              - Mejora en la ventana de definicin de listados.
 *              - Pequeos cambios en el formato del listado.
 *              - Actualizacin del soporte multilenguaje.
 *      Jun 03  - Pruebas de la versin 1.5
 *              - Se implementan las nuevas opciones de la librera de Ryszard Rylko
 *              - Se implementa el filtrado de la base de datos.
 *      Ago 03  - Se corrige bug en establecimiento de filtro.
 *              - Actualizado soporte para italiano (Arcangelo Molinaro).
 *              - Actualizado soporte multilenguage.
 *              - Actualizado el soporte para filtrado.
 *      Sep 03  - Idioma Vasco listo. Gracias a Gerardo Fernndez.
 *              - Idioma Italaino listo. Gracias a Arcangelo Molinaro.
 *              - Idioma Francs listo. Gracias a Chris Jouniauxdiv.
 *              - Idioma Polaco listo. Gracias a Jacek Kubica.
 *      Oct 03  - Solucionado problema con las clausulas ON FIND y ON PRINT, ahora
 *                ya tienen el efecto deseado. Gracias a Grigory Filiatov.
 *              - Se cambia la referencia a _ExtendedNavigation por _HMG_SYSDATA [ 255 ]
 *                para adecuarse a la sintaxis de la construcin 76.
 *              - Idioma Alemn listo. Gracias a Andreas Wiltfang.
 *      Nov 03  - Problema con dbs en set exclusive. Gracias a cas_HMG.
 *              - Problema con tablas con pocos campos. Gracias a cas_HMG.
 *              - Cambio en demo para ajustarse a nueva sintaxis RDD Harbour (DBFFPT).
 *      Dic 03  - Ajuste de la longitud del control para fecha. Gracias a Laszlo Henning.
 *      Ene 04  - Problema de bloqueo con SET DELETED ON. Gracias a Grigory Filiatov y Roberto Lpez.
 *
 *
 * - Limitaciones -
 * ================
 *      - No se pueden realizar busquedas por campos lgico o memo.
 *      - No se pueden realizar busquedas en indices con claves compuestas, la busqueda
 *        se realiza por el primer campo de la clave compuesta.
 *
 *
 * - Por hacer -
 * =============
 *      - Implementar busqueda del siguiente registro.
 *
 *
*/


MEMVAR _HMG_SYSDATA
MEMVAR _HMG_CMACROTEMP

// Ficheros de definiciones.---------------------------------------------------
#include "hmg.ch"
#include "dbstruct.ch"

// Declaracin de definiciones.------------------------------------------------
// Generales.
#define ABM_CRLF                HB_OsNewLine()

// Estructura de la etiquetas.
#define ABM_LBL_LEN             5
#define ABM_LBL_NAME            1
#define ABM_LBL_ROW             2
#define ABM_LBL_COL             3
#define ABM_LBL_WIDTH           4
#define ABM_LBL_HEIGHT          5

// Estructura de los controles de edicin.
#define ABM_CON_LEN             7
#define ABM_CON_NAME            1
#define ABM_CON_ROW             2
#define ABM_CON_COL             3
#define ABM_CON_WIDTH           4
#define ABM_CON_HEIGHT          5
#define ABM_CON_DES             6
#define ABM_CON_TYPE            7

// Tipos de controles de edicin.
#define ABM_TEXTBOXC            1
#define ABM_TEXTBOXN            2
#define ABM_DATEPICKER          3
#define ABM_CHECKBOX            4
#define ABM_EDITBOX             5
#define ABM_COMBOBOX            6   //added 20170214

// Estructura de las opciones de usuario.
#define ABM_OPC_TEXTO           1
#define ABM_OPC_BLOQUE          2

// Tipo de accin al definir las columnas del listado.
#define ABM_LIS_ADD             1
#define ABM_LIS_DEL             2

// Tipo de accin al definir los registros del listado.
#define ABM_LIS_SET1            1
#define ABM_LIS_SET2            2

// Declaracin de variables globales.------------------------------------------
static _cArea           as character            // Nombre del area de la bdd.
static _aEstructura     as array                // Estructura de la bdd.
static _aIndice         as array                // Nombre de los indices de la bdd.
static _aIndiceCampo    as array                // Nmero del campo indice.
static _nIndiceActivo   as array                // Indice activo.
static _aNombreCampo    as array                // Nombre desciptivo de los campos de la bdd.
static _aEditable       as array                // Indicador de si son editables.
static _cTitulo         as character            // Ttulo de la ventana.
static _nAltoPantalla   as numeric              // Alto de la pantalla.
static _nAnchoPantalla  as numeric              // Ancho de la pantalla.
static _aEtiqueta       as array                // Datos de las etiquetas.
static _aControl        as array                // Datos de los controles.
static _aCampoTabla     as array                // Nombre de los campos para la tabla.
static _aAnchoTabla     as array                // Anchos de los campos para la tabla.
static _aCabeceraTabla  as array                // Texto de las columnas de la tabla.
static _aAlineadoTabla  as array                // Alineacin de las columnas de la tabla.
static _aVisibleEnTabla as array                // Campos visibles en la tabla.
static _nControlActivo  as numeric              // Control con foco.
static _aOpciones       as array                // Opciones del usuario.
static _bGuardar        as codeblock            // Accin para guardar registro.
static _bBuscar         as codeblock            // Accin para buscar registro.
static _bImprimir       as codeblock            // Accin para imprimir listado.
static _lFiltro         as logical              // Indicativo de filtro activo.
static _cFiltro         as character            // Condicin de filtro.

static _cArrDescrip     as character            // Name of the array descript file  *.arr
static _aEditConfig     as array                // Estructura de la bdd. + Labels + Controls  //add 20151217

/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3()
 * Descripcin: Funcin inicial. Comprueba los parmetros pasados, crea la estructura
 *              para las etiquetas y controles de edicin y crea la ventana de visualizacin
 *              de registro.
 *  Parmetros: cArea           Nombre del area de la bdd. Por defecto se toma el area
 *                              actual.
 *              cTitulo         Ttulo de la ventana de edicin. Por defecto se toma el
 *                              nombre de la base de datos actual.
 *              aNombreCampo    Matriz de valores carcter con los nombres descriptivos de
 *                              los campos de la bdd.
 *              aAvisoCampo     Matriz con los textos que se presentarn en la barra de
 *                              estado al editar o aadir un registro.
 *              aEditable       Matriz de valre lgicos que indica si el campo referenciado
 *                              esta activo en la ventana de edicin de registro.
 *              aVisibleEnTabla Matriz de valores lgicos que indica la visibilidad de los
 *                              campos del browse de la ventana de edicin.
 *              aOpciones       Matriz con los valores de las opciones de usuario.
 *              bGuardar        Bloque de cdigo para la accin de guardar registro.
 *              bBuscar         Bloque de cdigo para la accin de buscar registro.
 *              bImprimir       Bloque de cdigo para la accin imprimir.
 *    Devuelve: NIL
****************************************************************************************/
function ABM3( cArea, cTitulo, aNombreCampo, ;
          aAvisoCampo, aEditable, aVisibleEnTabla, ;
          aOpciones, bGuardar, bBuscar, ;
          bImprimir, cArrDescriptor, aComboBox )

////////// Declaracin de variables locales.-----------------------------------
   local   i              as numeric       // Indice de iteracin.
   local   j              as numeric       // Indice de iteracin.
   local   k              as numeric       // Indice de iteracin.
   local   nArea          as numeric       // Numero del area de la bdd.
   local   nRegistro      as numeric       // Nmero de regisrto de la bdd.
   local   lSalida        as logical       // Control de bucle.
   local   nVeces         as numeric       // Indice de iteracin.
   local   cIndice        as character     // Nombre del indice.
   local   cIndiceActivo  as character     // Nombre del indice activo.
   local   cClave         as character     // Clave del indice.
   local   nInicio        as numeric       // Inicio de la cadena de busqueda.
   local   nAnchoCampo    as numeric       // Ancho del campo actual.
   local   nAnchoEtiqueta as numeric       // Ancho mximo de las etiquetas.
   local   nFila          as numeric       // Fila de creacin del control de edicin.
   local   nColumna       as numeric       // Columna de creacin del control de edicin.
   local   aTextoOp       as numeric       // Texto de las opciones de usuario.
   local   _BakExtendedNavigation          // Estado de SET NAVIAGTION.
   Local _BackDeleted
   Local cFiltroAnt     as character     // Condicin del filtro anterior.
   public  nImpLen
     cArrDescrip := cArrDescriptor

////////// Guardar estado actual de SET DELETED y activarlo
   _BackDeleted := set( _SET_DELETED )
   SET DELETED ON

////////// Inicializacin del soporte multilenguaje.---------------------------
   InitMessages()

////////// Desactivacin de SET NAVIGATION.------------------------------------
   _BakExtendedNavigation := _HMG_SYSDATA [ 255 ]
   _HMG_SYSDATA [ 255 ]    := .F.

////////// Control de parmetros.----------------------------------------------

   // Area de la base de datos.
   if ( ValType( cArea ) != "C" ) .or. Empty( cArea )
      _cArea := Alias()
      if _cArea == ""
         msgExclamation( _HMG_SYSDATA [ 130 ][1], "EDIT EXTENDED" )
         return NIL
      endif
   else
      _cArea := cArea
   endif
   _aEstructura := (_cArea)->( dbStruct() )

*<< begin modif LEER Arrays
   lArrDescrip:= .F.  ;  nErrorCode := 0
   IF ( ValType( cArrDescrip ) = "C" ) .and. file( cArrDescrip )  // There is a Description file
         _aEditConfig := ft_RestArr( cArrDescrip, @nErrorCode )
      if nErrorCode = 0
         lArrDescrip := .T.                                        // Read it ok
      else
         MsgInfo ("Error restoring array: " + cArrDescrip)         // Error reading. Continue in compatibility mode
*        RETURN
      endif
   ENDIF
*>> end modif LEER Arrays

*<< begin modif ComboBox
   aCombos := aComboBox
   if !Empty(aCombos) .and. ( ValType(aCombos)="A" ) .and. ( HMG_LEN(aCombos)= HMG_LEN(_aEstructura) )
         for i := 1 to HMG_LEN( aCombos )
            if ValType( aCombos[i] ) = "C"
              aCombos[i] := AllTrim(aCombos[i])  //seria mejor eliminar todos los espacios?
               cTable := BeforAtNum("-",aCombos[i],1)
               cField := AfterAtNum (">",aCombos[i],1)
               if  &cTable->(FieldPos(cField) ) = 0
                  MsgBox("Field does not exist in:  "+ cTable,cField)
                  aCombos[i] := Nil
               endif
            elseif ValType( aCombos[i] ) = "A"               // es ListBox
               for j := 1 to HMG_LEN(aCombos[i])
                  if ValType( aCombos[i,j] ) != "C" .or. empty(aCombos[i,j])
                     aCombos[i,j] := Nil
                  endif
               next
            else
               aCombos[i] := Nil
            endif
         next
   else
      aCombos := array( HMG_LEN( _aEstructura ) )
   endif
   *>> end modif ComboBox


   // Ttulo de la ventana.
   if ( cTitulo == NIL )
      _cTitulo := _cArea
   else
      if ( Valtype( cTitulo ) != "C" )
         _cTitulo := _cArea
      else
         _cTitulo := cTitulo
      endif
   endif

   // Nombres de los campos.
   lSalida := .t.
   if ( ValType( aNombreCampo ) != "A" )
      lSalida := .f.
   else
      if ( HMG_LEN( aNombreCampo ) != HMG_LEN( _aEstructura ) )
         lSalida := .f.
      else
         for i := 1 to HMG_LEN( aNombreCampo )
            if ValType( aNombreCampo[i] ) != "C"
               lSalida := .f.
               exit
            endif
         next
      endif
   endif
   if lSalida
      _aNombreCampo := aNombreCampo
   else
      _aNombreCampo := {}
      for i := 1 to HMG_LEN( _aEstructura )
         aAdd( _aNombreCampo, HMG_UPPER( HB_ULEFT( _aEstructura[i,DBS_NAME], 1 ) ) + ;
                    HMG_LOWER( HB_USUBSTR( _aEstructura[i,DBS_NAME], 2 ) ) )
      next
   endif

   // Texto de aviso en la barra de estado de la ventana de edicin de registro.
   lSalida := .t.
   if ( ValType( aAvisoCampo ) != "A" )
      lSalida := .f.
   else
      if ( HMG_LEN( aAvisoCampo ) != HMG_LEN( _aEstructura ) )
         lSalida := .f.
      else
         for i := 1 to HMG_LEN( aAvisoCampo )
            if Valtype( aAvisoCampo[i] ) != "C"
               lSalida := .f.
               exit
            endif
         next
      endif
   endif
   if !lSalida
      aAvisoCampo := {}
      for i := 1 to HMG_LEN( _aEstructura )
         do case
            case _aEstructura[i,DBS_TYPE] == "C"
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][2] )
            case _aEstructura[i,DBS_TYPE] == "N"
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][3] )
            case _aEstructura[i,DBS_TYPE] == "D"
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][4] )
            case _aEstructura[i,DBS_TYPE] == "L"
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][5] )
            case _aEstructura[i,DBS_TYPE] == "M"
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][6] )
            otherwise
               aAdd( aAvisoCampo, _HMG_SYSDATA [ 130 ][7] )
         endcase
      next
   endif

   // Campos visibles en la tabla de la ventana de visualizacin de registros.
   lSalida := .t.
   if ( Valtype( aVisibleEnTabla ) != "A" )
      lSalida := .f.
   else
      if HMG_LEN( aVisibleEnTabla ) != HMG_LEN( _aEstructura )
         lSalida := .f.
      else
         for i := 1 to HMG_LEN( aVisibleEnTabla )
            if ValType( aVisibleEnTabla[i] ) != "L"
               lSalida := .f.
               exit
            endif
         next
      endif
   endif
   if lSalida
      _aVisibleEnTabla := aVisibleEnTabla
   else
      _aVisibleEnTabla := {}
      for i := 1 to HMG_LEN( _aEstructura )
         aAdd( _aVisibleEnTabla, .t. )
      next
   endif

   // Estado de los campos en la ventana de edicin de registro.
   lSalida := .t.
   if ( ValType( aEditable ) != "A" )
      lSalida := .f.
   else
      if HMG_LEN( aEditable ) != HMG_LEN( _aEstructura )
         lSalida := .f.
      else
         for i := 1 to HMG_LEN( aEditable )
            if ValType( aEditable[i] ) != "L"
               lSalida := .f.
               exit
            endif
         next
      endif
   endif
   if lSalida
      _aEditable := aEditable
   else
      _aEditable := {}
      for i := 1 to HMG_LEN( _aEstructura )
         aAdd( _aEditable, .T.)
      next
   endif

   // Opciones del usuario.
   lSalida := .t.

   if ValType( aOpciones ) != "A"
      lSalida := .f.
   elseif HMG_LEN(aOpciones)<1
      lSalida := .f.
   elseif HMG_LEN( aOpciones[1] ) != 2
      lSalida := .f.
   else
      for i := 1 to HMG_LEN( aOpciones )
         if ValType( aOpciones [i,ABM_OPC_TEXTO] ) != "C"
            lSalida := .f.
            exit
         endif
         if ValType( aOpciones [i,ABM_OPC_BLOQUE] ) != "B"
            lSalida := .f.
            exit
         endif
      next
   endif

   if lSalida
      _aOpciones := aOpciones
   else
      _aOpciones := {}
   endif

   // Accin al guardar.
   if ValType( bGuardar ) == "B"
      _bGuardar := bGuardar
   else
      _bGuardar := NIL
   endif

   // Accin al buscar.
   if ValType( bBuscar ) == "B"
      _bBuscar := bBuscar
   else
      _bBuscar := NIL
   endif

   // Accin al imprimir.
   if ValType( bImprimir ) == "B"
      _bImprimir := bImprimir
   else
      _bImprimir := NIL
   endif

////////// Seleccin del area de la bdd.---------------------------------------
   nRegistro     := (_cArea)->( RecNo() )
   nArea         := Select()
   cIndiceActivo := (_cArea)->( ordSetFocus() )
   cFiltroAnt    := (_cArea)->( dbFilter() )
   dbSelectArea( _cArea )
   (_cArea)->( dbGoTop() )

////////// Inicializacin de variables.----------------------------------------
   // Filtro.
   if (_cArea)->( dbFilter() ) == ""
      _lFiltro := .f.
   else
      _lFiltro := .t.
   endif
   _cFiltro := (_cArea)->( dbFilter() )

   // Indices de la base de datos.
   lSalida       := .t.
   k             := 1
   _aIndice      := {}
   _aIndiceCampo := {}
   nVeces        := 1
   aAdd( _aIndice, _HMG_SYSDATA [ 129 ][1] )
   aAdd( _aIndiceCampo, 0 )
   do while lSalida
      if ( (_cArea)->( ordName( k ) ) == "" )
         lSalida := .f.
      else
         cIndice := (_cArea)->( ordName( k ) )
         aAdd( _aIndice, cIndice )
         cClave := HMG_UPPER( (_cArea)->( ordKey( k ) ) )
         for i := 1 to HMG_LEN( _aEstructura )
            if nVeces <= 1
               nInicio := HB_UAT( _aEstructura[i,DBS_NAME], cClave )
               if  nInicio != 0
                  aAdd( _aIndiceCampo, i )
                  nVeces++
               endif
            endif
         next
      endif
      k++
      nVeces := 1
   enddo

   // Numero de indice.
   if ( (_cArea)->( ordSetFocus() ) == "" )
      _nIndiceActivo := 1
   else
      _nIndiceActivo := aScan( _aIndice, (_cArea)->( ordSetFocus() ) )
   endif

   // Tamao de la pantalla.
   _nAltoPantalla  := GETDESKTOPREALHEIGHT()
   _nAnchoPantalla := GETDESKTOPREALWIDTH()

*<< begin modif LEER Arrays
IF lArrDescrip                                           // There is a Description file
   _aEtiqueta   := _aEditConfig[2]                       // use it
   _aControl    := _aEditConfig[3]
   _aEditConfig := {}
*>> end modif LEER Arrays

ELSE    // calculate controls in standard way (1 col)
   // Datos de las etiquetas y los controles de la ventana de edicin.
   _aEtiqueta     := Array( HMG_LEN( _aEstructura ), ABM_LBL_LEN )
   _aControl      := Array( HMG_LEN( _aEstructura ), ABM_CON_LEN )
   nFila          := 10
   nColumna       := 10
   nAnchoEtiqueta := 0
   for i := 1 to HMG_LEN( _aNombreCampo )
      nAnchoEtiqueta := iif( nAnchoEtiqueta > ( HMG_LEN( _aNombreCampo[i] ) * 9 ),;
                   nAnchoEtiqueta,;
                   HMG_LEN( _aNombreCampo[i] ) * 9 )
   next

   * Determina la posicion de las etiquetas
   for i := 1 to HMG_LEN( _aEstructura )
      _aEtiqueta[i,ABM_LBL_NAME]   := "ABM3Etiqueta" + ALLTRIM( STR( i ,4,0) )
      _aEtiqueta[i,ABM_LBL_ROW]    := nFila
      _aEtiqueta[i,ABM_LBL_COL]    := nColumna
      _aEtiqueta[i,ABM_LBL_WIDTH]  := HMG_LEN( _aNombreCampo[i] ) * 9
      _aEtiqueta[i,ABM_LBL_HEIGHT] := 25
   * Define los controles
      do case
         case _aEstructura[i,DBS_TYPE] == "C"
            _aControl[i,ABM_CON_NAME]   := "ABM3Control" + ALLTRIM( STR( i ,4,0) )
            _aControl[i,ABM_CON_ROW]    := nFila
            _aControl[i,ABM_CON_COL]    := nColumna + nAnchoEtiqueta +20
               IF (_aEstructura[i,DBS_LEN] * 9) < 50
                  _aControl[i,ABM_CON_WIDTH] := 50
               ELSEIF (_aEstructura[i,DBS_LEN] * 9) > 270    // added
                  _aControl[i,ABM_CON_WIDTH] := 270
               ELSE
                  _aControl[i,ABM_CON_WIDTH] := _aEstructura[i,DBS_LEN] * 9
               ENDIF
            _aControl[i,ABM_CON_HEIGHT] := 25
            _aControl[i,ABM_CON_DES]    := aAvisoCampo[i]
            _aControl[i,ABM_CON_TYPE]   := ABM_TEXTBOXC
            nFila += 35
         case _aEstructura[i,DBS_TYPE] == "D"
            _aControl[i,ABM_CON_NAME]   := "ABM3Control" + ALLTRIM( STR( i ,4,0) )
            _aControl[i,ABM_CON_ROW]    := nFila
            _aControl[i,ABM_CON_COL]    := nColumna + nAnchoEtiqueta + 20
            _aControl[i,ABM_CON_WIDTH]  := _aEstructura[i,DBS_LEN] * 9
            _aControl[i,ABM_CON_HEIGHT] := 25
            _aControl[i,ABM_CON_DES]    := aAvisoCampo[i]
            _aControl[i,ABM_CON_TYPE]   := ABM_DATEPICKER
            nFila += 35
         case _aEstructura[i,DBS_TYPE] == "N"
            _aControl[i,ABM_CON_NAME]   := "ABM3Control" + ALLTRIM( STR( i ,4,0) )
            _aControl[i,ABM_CON_ROW]    := nFila
            _aControl[i,ABM_CON_COL]    := nColumna + nAnchoEtiqueta + 20
            _aControl[i,ABM_CON_WIDTH]  := iif( ( _aEstructura[i,DBS_LEN] * 9 ) < 50, 50, _aEstructura[i,DBS_LEN] * 10 )
            _aControl[i,ABM_CON_HEIGHT] := 25
            _aControl[i,ABM_CON_DES]    := aAvisoCampo[i]
            _aControl[i,ABM_CON_TYPE]   := ABM_TEXTBOXN
            nFila += 35
         case _aEstructura[i,DBS_TYPE] == "L"
            _aControl[i,ABM_CON_NAME]   := "ABM3Control" + ALLTRIM( STR( i ,4,0) )
            _aControl[i,ABM_CON_ROW]    := nFila
            _aControl[i,ABM_CON_COL]    := nColumna + nAnchoEtiqueta + 20
            _aControl[i,ABM_CON_WIDTH]  := 25
            _aControl[i,ABM_CON_HEIGHT] := 25
            _aControl[i,ABM_CON_DES]    := aAvisoCampo[i]
            _aControl[i,ABM_CON_TYPE]   := ABM_CHECKBOX
            nFila += 35
         case _aEstructura[i,DBS_TYPE] == "M"
            _aControl[i,ABM_CON_NAME]   := "ABM3Control" + ALLTRIM( STR( i ,4,0) )
            _aControl[i,ABM_CON_ROW]    := nFila
            _aControl[i,ABM_CON_COL]    := nColumna + nAnchoEtiqueta + 20
            _aControl[i,ABM_CON_WIDTH]  := 300
            _aControl[i,ABM_CON_HEIGHT] := 70
            _aControl[i,ABM_CON_DES]    := aAvisoCampo[i]
            _aControl[i,ABM_CON_TYPE]   := ABM_EDITBOX
            nFila += 80
      endcase
   next
ENDIF    // calcular los controles

   // Datos de la tabla de la ventana de visualizacin.
   _aCampoTabla    := {}
   _aAnchoTabla    := {}
   _aCabeceraTabla := {}
   _aAlineadoTabla := {}
   for i := 1 to HMG_LEN( _aEstructura )
      if _aVisibleEnTabla[i]
         aAdd( _aCampoTabla, _cArea + "->" + _aEstructura[i, DBS_NAME] )
         nAnchoCampo    := iif( ( _aEstructura[i,DBS_LEN] * 10 ) < 50,   ;
                   50,                                    ;
                   _aEstructura[i,DBS_LEN] * 10 )
         nAnchoEtiqueta := HMG_LEN( _aNombreCampo[i] ) * 10
         aAdd( _aAnchoTabla, iif( nAnchoEtiqueta > nAnchoCampo,          ;
                   nAnchoEtiqueta,                        ;
                   nAnchoCampo ) )
         aAdd( _aCabeceraTabla, _aNombreCampo[i] )
         aAdd( _aAlineadoTabla, iif( _aEstructura[i,DBS_TYPE] == "N",     ;
                      BROWSE_JTFY_RIGHT,                   ;
                      BROWSE_JTFY_LEFT ) )
      endif
   next

////////// Definicin de la ventana de visualizacin.--------------------------
   define window wndABM3Edit               ;
      AT 60, 30                       ;
      width _nAnchoPantalla - 60      ;
      height _nAltoPantalla - 140     ;
      title _cTitulo                  ;
      modal                           ;
      nosize                          ;
      nosysmenu                       ;
      on init {|| ABM3Redibuja() }    ;
      on release {|| ABM3salir(nRegistro, cIndiceActivo, cFiltroAnt, nArea) }   ;
      font "ms sans serif" size 9

      // Define la barra de estado de la ventana de visualizacin.
      define statusbar font "ms sans serif" size 9
         statusitem _HMG_SYSDATA [ 129 ][19]                           // 1
         statusitem _HMG_SYSDATA [ 129 ][20]         width 100 raised  // 2
         statusitem _HMG_SYSDATA [ 129 ][2] +': '     width 200 raised // 3
      end statusbar

      // Define la barra de botones de la ventana de visualizacin.
      define toolbar tbEdit buttonsize 90, 32 flat righttext border
         button tbbCerrar  caption _HMG_SYSDATA [ 128 ][1]   ;
                 picture "HMG_EDIT_CLOSE"          ;
                 action  wndABM3Edit.Release
         button tbbNuevo   caption _HMG_SYSDATA [ 128 ][2]               ;
                 picture "HMG_EDIT_NEW"            ;
                 action  {|| ABM3Editar( .t. ) }
         button tbbEditar  caption _HMG_SYSDATA [ 128 ][3]               ;
                 picture "HMG_EDIT_EDIT"           ;
                 action  {|| ABM3Editar( .f. ) }
         button tbbBorrar  caption _HMG_SYSDATA [ 128 ][4]               ;
                 picture "HMG_EDIT_DELETE"         ;
                 action  {|| ABM3Borrar() }
         button tbbBuscar  caption _HMG_SYSDATA [ 128 ][5]               ;
                 picture "HMG_EDIT_FIND"           ;
                 action  {|| ABM3Buscar() }
         button tbbListado caption _HMG_SYSDATA [ 128 ][6]               ;
                 picture "HMG_EDIT_PRINT"          ;
                 action  {|| ABM3Imprimir() }
      end toolbar

   end window

////////// Creacin de los controles de la ventana de visualizacin.-----------
   @ 45, 10 frame frmEditOpciones          ;
      of wndABM3Edit                  ;
      caption ""                      ;
      width wndABM3Edit.Width - 25    ;
      height 65
   @ 112, 10 frame frmEditTabla            ;
      of wndABM3Edit                  ;
      caption ""                      ;
      width wndABM3Edit.Width - 25    ;
      height wndABM3Edit.Height - 165
    @ 60, 20 label lblIndice               ;
      of wndABM3Edit                  ;
      value _HMG_SYSDATA [ 130 ] [26]            ;
      width 150                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 75, 20 combobox cbIndices                     ;
      of wndABM3Edit                          ;
      items _aIndice                          ;
      value _nIndiceActivo                    ;
      width 150                               ;
      font "arial" size 9                     ;
      on change {|| ABM3CambiarOrden() }
   nColumna := wndABM3Edit.Width - 175
   aTextoOp := {}
   for i := 1 to HMG_LEN( _aOpciones )
      aAdd( aTextoOp, _aOpciones[i,ABM_OPC_TEXTO] )
   next
   @ 60, nColumna label lblOpciones        ;
      of wndABM3Edit                  ;
      value _HMG_SYSDATA [ 129 ] [5]            ;
      width 150                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 75, nColumna combobox cbOpciones              ;
      of wndABM3Edit                          ;
      items aTextoOp                          ;
      value 1                                 ;
      width 150                               ;
      font "arial" size 9                     ;
      on change {|| ABM3EjecutaOpcion() }
   @ 65, (wndABM3Edit.Width / 2)-110 button btnFiltro1     ;
      of wndABM3Edit                                  ;
      caption _HMG_SYSDATA [ 128 ][10]                       ;
      action {|| ABM3ActivarFiltro() }                ;
      width 100                                       ;
      height 32                                       ;
      font "ms sans serif" size 9
   @ 65, (wndABM3Edit.Width / 2)+5 button btnFiltro2       ;
      of wndABM3Edit                                  ;
      caption _HMG_SYSDATA [ 128 ][11]                    ;
      action {|| ABM3DesactivarFiltro() }             ;
      width 100                                       ;
      height 32                                       ;
      font "ms sans serif" size 9
   @ 132, 20 browse brwABM3Edit                                                    ;
      of wndABM3Edit                                                          ;
      width wndABM3Edit.Width - 45                                            ;
      height wndABM3Edit.Height - 195                                         ;
      headers _aCabeceraTabla                                                 ;
      widths _aAnchoTabla                                                     ;
      workarea &_cArea                                                        ;
      fields _aCampoTabla                                                     ;
      value ( _cArea)->( RecNo() )                                            ;
      font "arial" size 9                                                     ;
      on change {|| (_cArea)->( dbGoto( wndABM3Edit.brwABM3Edit.Value ) ),    ;
               ABM3Redibuja( .f. ) }                                     ;
      on dblclick ABM3Editar( .f. )                                           ;
      justify _aAlineadoTabla

   // Comprueba el estado de las opciones de usuario.
   if HMG_LEN( _aOpciones ) == 0
      wndABM3Edit.cbOpciones.Enabled := .f.
   endif

////////// Activacin de la ventana de visualizacin.--------------------------
   activate window wndABM3Edit

////////// Restauracin de SET NAVIGATION.-------------------------------------
   _HMG_SYSDATA [ 255 ] := _BakExtendedNavigation

////////// Restaurar SET DELETED a su valor inicial

   set( _SET_DELETED , _BackDeleted  )

return NIL


/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3salir()
 * Descripcin: Cierra la ventana de visualizacin de registros y sale.
 *  Parmetros: Ninguno.
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3salir( nRegistro, cIndiceActivo, cFiltroAnt, nArea )

////////// Declaracin de variables locales.-----------------------------------
   local bFiltroAnt as codeblock           // Bloque de cdigo del filtro.

////////// Inicializacin de variables.----------------------------------------
   bFiltroAnt := iif( Empty( cFiltroAnt ),;
            &("{||NIL}"),;
            &("{||" + cFiltroAnt + "}") )

////////// Restaura el area de la bdd inicial.---------------------------------
   (_cArea)->( dbGoTo( nRegistro ) )
   (_cArea)->( ordSetFocus( cIndiceActivo ) )
   (_cArea)->( dbSetFilter( bFiltroAnt, cFiltroAnt ) )
   dbSelectArea( nArea )

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Redibuja()
 * Descripcin: Actualizacin de la ventana de visualizacin de registros.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3Redibuja( lTabla )

////////// Control de parmetros.----------------------------------------------
   if ValType( lTabla ) != "L"
      lTabla := .f.
   endif

////////// Refresco de la barra de botones.------------------------------------
   if (_cArea)->( RecCount() ) == 0
      wndABM3Edit.tbbEditar.Enabled  := .f.
      wndABM3Edit.tbbBorrar.Enabled  := .f.
      wndABM3Edit.tbbBuscar.Enabled  := .f.
      wndABM3Edit.tbbListado.Enabled := .f.
   else
      wndABM3Edit.tbbEditar.Enabled  := .t.
      wndABM3Edit.tbbBorrar.Enabled  := .t.
      wndABM3Edit.tbbBuscar.Enabled  := .t.
      wndABM3Edit.tbbListado.Enabled := .t.
   endif

////////// Refresco de la barra de estado.-------------------------------------
   wndABM3Edit.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 129 ][19] + _cFiltro
   wndABM3Edit.StatusBar.Item( 2 ) := _HMG_SYSDATA [ 129 ][20] + iif( _lFiltro, _HMG_SYSDATA [ 130 ][29], _HMG_SYSDATA [ 130 ][30] )
   wndABM3Edit.StatusBar.Item( 3 ) := _HMG_SYSDATA [ 129 ][2] + ': '+                                  ;
                  ALLTRIM( STR( (_cArea)->( RecNo() ) ) ) + "/" + ;
                  ALLTRIM( STR( (_cArea)->( RecCount() ) ) )


////////// Refresca el browse si se indica.
   if lTabla
      wndABM3Edit.brwABM3Edit.Value := (_cArea)->( RecNo() )
      wndABM3Edit.brwABM3Edit.Refresh
   endif

////////// Coloca el foco en el browse.
   wndABM3Edit.brwABM3Edit.SetFocus

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3CambiarOrden()
 * Descripcin: Cambia el orden activo.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3CambiarOrden()

////////// Declaracin de variables locales.-----------------------------------
   local cIndice as character              // Nombre del indice.
   local nIndice as numeric                // Nmero del indice.

////////// Inicializa las variables.-------------------------------------------
   nIndice := wndABM3Edit.cbIndices.Value
   cIndice := wndABM3Edit.cbIndices.Item( nIndice )

////////// Cambia el orden del area de trabajo.--------------------------------
   nIndice--
   (_cArea)->( ordSetFocus( nIndice ) )
   // (_cArea)->( dbGoTop() )
   nIndice++
   _nIndiceActivo := nIndice
   ABM3Redibuja( .t. )

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3EjecutaOpcion()
 * Descripcin: Ejecuta las opciones del usuario.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3EjecutaOpcion()

////////// Declaracin de variables locales.-----------------------------------
   local nItem    as numeric               // Numero del item seleccionado.
   local bBloque  as codebloc              // Bloque de codigo a ejecutar.

////////// Inicializacin de variables.----------------------------------------
   nItem   := wndABM3Edit.cbOpciones.Value
   bBloque := _aOpciones[nItem,ABM_OPC_BLOQUE]

////////// Ejecuta la opcin.--------------------------------------------------
   Eval( bBloque )

////////// Refresca el browse.
   ABM3Redibuja( .t. )

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Editar( lNuevo )
 * Descripcin: Creacin de la ventana de edicin de registro.
 *  Parmetros: lNuevo          Valor lgico que indica si se est aadiendo un registro
 *                              o editando el actual.
 *    Devuelve:
****************************************************************************************/
static function ABM3Editar( lNuevo )

////////// Declaracin de variables locales.-----------------------------------
   local i              as numeric         // Indice de iteracin.
   local nAnchoEtiqueta as numeric         // Ancho mximo de las etiquetas.
   local nAltoControl   as numeric         // Alto total de los controles de edicin.
   local nAncho         as numeric         // Ancho de la ventana de edicin .
   local nAlto          as numeric         // Alto de la ventana de edicin.
   local nAnchoTope     as numeric         // Ancho mximo de la ventana de edicin.
   local nAltoTope      as numeric         // Alto mximo de la ventana de edicin.
   local nAnchoSplit    as numeric         // Ancho de la ventana Split.
   local nAltoSplit     as numeric         // Alto de la ventana Split.
   local cTitulo        as character       // Ttulo de la ventana.
   local cMascara       as array           // Mscara de edicin de los controles numricos.
   local nAnchoContr    as numeric

////////// Control de parmetros.----------------------------------------------
   if ( ValType( lNuevo ) != "L" )
      lNuevo := .t.
   endif

////////// Incializacin de variables.-----------------------------------------
   nAnchoEtiqueta := 0
   nAnchoControl  := 0
   nAltoControl   := 0
IF lArrDescrip
   Wider := .F.
   aMedidas := Array (12)
   aFill (aMedidas, 0)
*<< begin modif Columnas
   FOR i := 1 to HMG_LEN( _aEstructura )
      IF _aControl [i,8] = 1     //8=ABM_CON_CC  (column)
*          increm alto etiq 1
         aMedidas[1] += _aEtiqueta [i,ABM_LBL_HEIGHT] +10
*          increm alto contrs 1
         aMedidas[2] += _aControl  [i,ABM_CON_HEIGHT] +10
*           elija ancho etiq mayor col 1
         aMedidas[3] := iif( aMedidas[3] > _aEtiqueta [i,ABM_LBL_WIDTH],   ;
         aMedidas[3],                      _aEtiqueta [i,ABM_LBL_WIDTH] )
*           elija ancho contrs mayor col 1
         aMedidas[4] := iif( aMedidas[4] > _aControl [i,ABM_CON_WIDTH],    ;
         aMedidas[4],                      _aControl [i,ABM_CON_WIDTH] )

      ELSEIF _aControl [i,8] = 2     //8=ABM_CON_CC
*           increm alto etiq 2
         aMedidas[5] += _aEtiqueta [i,ABM_LBL_HEIGHT] +10
*           increm alto contrs 2
         aMedidas[6] += _aControl  [i,ABM_CON_HEIGHT] +10
*           elija ancho etiq mayor col 2
         aMedidas[7] := iif( aMedidas[7] > _aEtiqueta [i,ABM_LBL_WIDTH],    ;
         aMedidas[7],                      _aEtiqueta [i,ABM_LBL_WIDTH] )
*           elija ancho contrs mayor col 2
         aMedidas[8] := iif( aMedidas[8] > _aControl [i,ABM_CON_WIDTH],     ;
         aMedidas[8],                      _aControl [i,ABM_CON_WIDTH] )

      ELSEIF _aControl [i,8] = 3
         Wider := .T.
*           increm alto etiq 3
         aMedidas[9] += _aEtiqueta [i,ABM_LBL_HEIGHT] +10
*           increm alto contrs 3
         aMedidas[10] += _aControl [i,ABM_CON_HEIGHT] +10
*           elija ancho etiq mayor col 3
         aMedidas[11] := iif( aMedidas[11] > _aEtiqueta [i,ABM_LBL_WIDTH],   ;
         aMedidas[11],                       _aEtiqueta [i,ABM_LBL_WIDTH] )
*           elija ancho contrs mayor col 3
         aMedidas[12] := iif( aMedidas[12] > _aControl [i,ABM_CON_WIDTH],    ;
         aMedidas[12],                       _aControl [i,ABM_CON_WIDTH]  )
      ENDIF
   NEXT
         aMedidas[4]  := iif( aMedidas[4]  >  300, 300, aMedidas[4] )
         aMedidas[8]  := iif( aMedidas[8]  >  300, 300, aMedidas[8] )

*     Buscar el mayor alto-maximo   (etiq y cntr) -->  alto maximo
   nAltoControl := iif( aMedidas[1]  >  aMedidas[2],   aMedidas[1], aMedidas[2])
   nAltoControl := iif( aMedidas[5]  >  nAltoControl,  aMedidas[5], nAltoControl)
   nAltoControl := iif( aMedidas[6]  >  nAltoControl,  aMedidas[6], nAltoControl)
   nAltoControl := iif( aMedidas[9]  >  nAltoControl,  aMedidas[9], nAltoControl)
   nAltoControl := iif( aMedidas[10] >  nAltoControl,  aMedidas[10], nAltoControl)
   nAltoControl += 10
*     Sumar todos los ancho-maximos (etiq y cntr)
   nAnchoEtiqueta := aMedidas[3] + aMedidas[7] + aMedidas[11]
   nAnchoControl  := aMedidas[4] + aMedidas[8] + aMedidas[12]
   nAnchoControl += iif(wider,160,70)
*>> end modif Columnas

ELSE     // legacy
   for i := 1 to HMG_LEN( _aEtiqueta )
      nAnchoEtiqueta := iif( nAnchoEtiqueta > _aEtiqueta[i,ABM_LBL_WIDTH],;
                   nAnchoEtiqueta,;
                   _aEtiqueta[i,ABM_LBL_WIDTH] )
      nAnchoControl  := iif( nAnchoControl > _aControl[i,ABM_CON_WIDTH],;
                   nAnchoControl,;
                   _aControl[i,ABM_CON_WIDTH] )
      nAltoControl   += _aControl[i,ABM_CON_HEIGHT] + 10
   next
ENDIF
   nAltoSplit  := 10 + nAltoControl + 10 + 15
   nAnchoSplit := 10 + nAnchoEtiqueta + 10 + nAnchoControl + 10 + 15
   nAlto       := 15 + 65 + nAltoSplit + 15
   nAltoTope   := _nAltoPantalla - 130
   nAncho      := 15 + nAnchoSplit + 15
   nAncho      := iif( nAncho < 300, 300, nAncho )
   nAnchoTope  := _nAnchoPantalla - 60
   cTitulo     := iif( lNuevo, _HMG_SYSDATA [ 129 ][6], _HMG_SYSDATA [ 129 ][7] )

////////// Define la ventana de edicin de registro.---------------------------
   nAlto  := iif( nAlto > nAltoTope, nAltoTope, nAlto )
   nAncho := iif( nAncho > nAnchoTope, nAnchoTope, nAncho )
   define window wndABM3EditNuevo                             ;
      AT (_nAltoPantalla-nAlto)/2, (_nAnchoPantalla-nAncho)/2 ;
      width nAncho                                            ;
      height  nAlto                                           ;
      title cTitulo                                           ;
      modal                                                   ;
      nosize                                                  ;
      nosysmenu                                               ;
      font "ms sans serif" size 9

      // Define la barra de estado de la ventana de edicin de registro.
      define statusbar font "ms sans serif" size 9
         statusitem ""
      end statusbar

      define splitbox

         // Define la barra de botones de la ventana de edicin de registro.
         define toolbar tbEditNuevo buttonsize 90, 32 flat righttext
            button tbbCancelar caption _HMG_SYSDATA [ 128 ][7]              ;
                     picture "HMG_EDIT_CANCEL"        ;
                     action  wndABM3EditNuevo.Release
            button tbbAceptar  caption _HMG_SYSDATA [ 128 ][8]              ;
                     picture "HMG_EDIT_OK"            ;
                     action  ABM3EditarGuardar( lNuevo )
            button tbbCopiar   caption _HMG_SYSDATA [ 128 ][9]              ;
                     picture "HMG_EDIT_COPY"          ;
                     action  ABM3EditarCopiar()
         end toolbar

         // Define la ventana donde van contenidos los controles de edicin.
         define window wndABM3EditNuevoSplit             ;
            width iif( nAncho > nAnchoTope,         ;
                  nAnchoTope - 10,             ;
                  nAnchoSplit  - 1 )           ;
            height iif( nAlto > nAltoTope,          ;
                   nAltoTope - 95,             ;
                   nAltoSplit - 1 )            ;
            virtual width nAnchoSplit               ;
            virtual height nAltoSplit               ;
            splitchild                              ;
            nocaption                               ;
            font "ms sans serif" size 9             ;
            focused
         end window
      end splitbox
   end window

////////// Define las etiquetas de los controles.------------------------------
   for i := 1 to HMG_LEN( _aEtiqueta )

      _HMG_cMacroTemp := _aEtiqueta[i,ABM_LBL_NAME]

      @ _aEtiqueta[i,ABM_LBL_ROW], _aEtiqueta[i,ABM_LBL_COL]  ;
         label &_HMG_cMacroTemp ;
         of wndABM3EditNuevoSplit                        ;
         value _aNombreCampo[i]                          ;
         width _aEtiqueta[i,ABM_LBL_WIDTH]               ;
         height _aEtiqueta[i,ABM_LBL_HEIGHT]             ;
         font "ms sans serif" size 9
   next

////////// Define los controles de edicin.------------------------------------
   for i := 1 to HMG_LEN( _aControl )
      do case
*<< begin modif ListBox  /  ComboBox
         case _aControl[i,ABM_CON_TYPE] == ABM_TEXTBOXC .or. _aControl[i,ABM_CON_TYPE] == ABM_COMBOBOX
         cFuncName := "LB_" + _cArea + "_" + _aEstructura[i, DBS_NAME]
            IF valtype( aCombos[i] ) = "C"      // it is ComboBox
               _aControl[i,ABM_CON_TYPE]   := ABM_COMBOBOX
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                       ComboBox &_HMG_cMacroTemp                       ;
                       of wndABM3EditNuevoSplit                        ;
                       ITEMSOURCE  &( aCombos[i] )                  ;
                       VALUESOURCE &( aCombos[i] )                  ;
                       height _aControl[i,ABM_CON_HEIGHT]              ;
                       width  _aControl[i,ABM_CON_WIDTH]               ;
                       font "arial" size 9                             ;
                       on gotfocus ABM3ConFoco()                       ;
                       on lostfocus ABM3SinFoco()                      ;
                       on enter ABM3AlEntrar( )
            ELSEIF valtype( aCombos[i] ) = "A"     // it is ListBox
               aItems := aCombos[i]
               _aControl[i,ABM_CON_TYPE]   := ABM_COMBOBOX
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                       ComboBox &_HMG_cMacroTemp                       ;
                       of wndABM3EditNuevoSplit                        ;
                       ITEMS aItems                                    ;
                       value 1                                         ;
                       height _aControl[i,ABM_CON_HEIGHT]              ;
                       width  _aControl[i,ABM_CON_WIDTH]               ;
                       font "arial" size 9                             ;
                       on gotfocus ABM3ConFoco()                       ;
                       on lostfocus ABM3SinFoco()                      ;
                       on enter ABM3AlEntrar( )
            
            ELSEIF hb_IsFunction(cFuncName)     // it is Function
               aItems := &(cFuncName + "()")
               _aControl[i,ABM_CON_TYPE]   := ABM_COMBOBOX
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                       ComboBox &_HMG_cMacroTemp                       ;
                       of wndABM3EditNuevoSplit                        ;
                       ITEMS aItems                                    ;
                       value 1                                         ;
                       height _aControl[i,ABM_CON_HEIGHT]              ;
                       width  _aControl[i,ABM_CON_WIDTH]               ;
                       font "arial" size 9                             ;
                       on gotfocus ABM3ConFoco()                       ;
                       on lostfocus ABM3SinFoco()                      ;
                       on enter ABM3AlEntrar( )
   *>> end modif ListBox
            ELSE                                // It is a Normal TextBox
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                     textbox &_HMG_cMacroTemp                        ;
                     of wndABM3EditNuevoSplit                        ;
                     value ""                                        ;
                     height _aControl[i,ABM_CON_HEIGHT]              ;
                     width _aControl[i,ABM_CON_WIDTH] + 25           ;
                     font "arial" size 9                             ;
                     maxlength _aEstructura[i,DBS_LEN]               ;
                     on gotfocus ABM3ConFoco()                       ;
                     on lostfocus ABM3SinFoco()                      ;
                     on enter ABM3AlEntrar( )
            ENDIF
         case _aControl[i,ABM_CON_TYPE] == ABM_DATEPICKER
            _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
            @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
               datepicker &_HMG_cMacroTemp           ;
               of wndABM3EditNuevoSplit                        ;
               height _aControl[i,ABM_CON_HEIGHT]              ;
               width _aControl[i,ABM_CON_WIDTH] + 25           ;
               font "arial" size 9                             ;
               SHOWNONE                ;
               on gotfocus ABM3ConFoco()                       ;
               on lostfocus ABM3SinFoco()
         case _aControl[i,ABM_CON_TYPE] == ABM_TEXTBOXN
            if ( _aEstructura[i,DBS_DEC] == 0 )
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                  textbox &_HMG_cMacroTemp           ;
                  of wndABM3EditNuevoSplit                        ;
                  value ""                                        ;
                  height _aControl[i,ABM_CON_HEIGHT]              ;
                  width _aControl[i,ABM_CON_WIDTH]                ;
                  numeric                                         ;
                  font "arial" size 9                             ;
                  maxlength _aEstructura[i,DBS_LEN]               ;
                  on gotfocus ABM3ConFoco( i )                    ;
                  on lostfocus ABM3SinFoco( i )                   ;
                  on enter ABM3AlEntrar()
            else
               cMascara := ""
               cMascara := REPLICATE( "9", _aEstructura[i,DBS_LEN] -   ;
                         ( _aEstructura[i,DBS_DEC] + 1 ) )
               cMascara += "."
               cMascara += REPLICATE( "9", _aEstructura[i,DBS_DEC] )
               _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
               @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
                  textbox &_HMG_cMacroTemp              ;
                  of wndABM3EditNuevoSplit                        ;
                  value ""                                        ;
                  height _aControl[i,ABM_CON_HEIGHT]              ;
                  width _aControl[i,ABM_CON_WIDTH]                ;
                  numeric                                         ;
                  inputmask cMascara                              ;
                  on gotfocus ABM3ConFoco()                       ;
                  on lostfocus ABM3SinFoco()                      ;
                  on enter ABM3AlEntrar()
            endif
         case _aControl[i,ABM_CON_TYPE] == ABM_CHECKBOX
            _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
            @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
               checkbox &_HMG_cMacroTemp             ;
               of wndABM3EditNuevoSplit                        ;
               caption ""                                      ;
               height _aControl[i,ABM_CON_HEIGHT]              ;
               width _aControl[i,ABM_CON_WIDTH]                ;
               value .f.                                       ;
               on gotfocus ABM3ConFoco()                       ;
               on lostfocus ABM3SinFoco()
         case _aControl[i,ABM_CON_TYPE] == ABM_EDITBOX
            _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
            @ _aControl[i,ABM_CON_ROW], _aControl[i,ABM_CON_COL]    ;
               editbox &_HMG_cMacroTemp              ;
               of wndABM3EditNuevoSplit                        ;
               width _aControl[i,ABM_CON_WIDTH]                ;
               height _aControl[i,ABM_CON_HEIGHT]              ;
               value ""                                        ;
               font "arial" size 9                             ;
               on gotfocus ABM3ConFoco()                       ;
               on lostfocus ABM3SinFoco()
      endcase
   next

////////// Actualiza los controles si se est editando.------------------------
   if !lNuevo
      for i := 1 to HMG_LEN( _aControl )
         _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
*<< begin modif ListBox / ComboBox
         IF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE] .and. valtype( aCombos[i] ) = "C"  // It is a ComboBox * (ValueSource)
            nRegistro := (cTable)->( RecNo() )                                           // ItemSource => Table related
            (cTable)->( dbGoTop() )
            Do While (cTable)->( !Eof() )
               cThisOne   := UPPER( alltrim((_cArea)->( FieldGet(i) ) ))  // this table
               cReferTable:= UPPER( alltrim( (cTable)->&cField ))         // related table
               if cReferTable = cThisOne
                  wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value := (cTable)->( RecNo() )
                  Exit                       // This is the saved value
               endif
               (cTable)->( DbSkip() )
            End
            (cTable)->( dbGoTo( nRegistro ) )
         ELSEIF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE]  .and. valtype( aCombos[i] ) = "A"   // It is a ListBox * (Item => aParamet)
            aItems := aCombos[i]  ;  cValor := RTRIM( (_cArea)->( FieldGet( i )) )
            nItem  := ASCAN(aItems, cValor )  // read the options list (for the user to select)
            wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value := nItem
            
         ELSEIF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE]                                // It is a ListBox * (Item => function list
            cFuncName := "LB_" + _cArea + "_" + _aEstructura[i, DBS_NAME] + "()"
            aItems := &(cFuncName)  ;  cValor := RTRIM( (_cArea)->( FieldGet( i )) )
            nItem  := ASCAN(aItems, cValor )  // read the options list (for the user to select)
            wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value := nItem
*>> end modif ListBox / ComboBox
         ELSE   //It is a normal TextBox
            wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value := (_cArea)->( FieldGet(i) )
         ENDIF
      next
   endif
////////// Establece el estado inicial de los controles.-----------------------
   for i := 1 to HMG_LEN( _aControl )
      _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
      wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Enabled := _aEditable[i]
   next

////////// Establece el estado del botn de copia.-----------------------------
   if !lNuevo
      wndABM3EditNuevo.tbbCopiar.Enabled := .f.
   endif

////////// Activa la ventana de edicin de registro.---------------------------
   center window wndABM3EditNuevo   // added 20160209
   activate window wndABM3EditNuevo

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3ConFoco()
 * Descripcin: Actualiza las etiquetas de los controles y presenta los mensajes en la
 *              barra de estado de la ventana de edicin de registro al obtener un
 *              control de edicin el foco.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3ConFoco()

////////// Declaracin de variables locales.-----------------------------------
   local i         as numeric              // Indice de iteracin.
   local cControl  as character            // Nombre del control activo.
   local acControl as array                // Matriz con los nombre de los controles.

////////// Inicializacin de variables.----------------------------------------
   cControl := This.Name
   acControl := {}
   for i := 1 to HMG_LEN( _aControl )
      aAdd( acControl, _aControl[i,ABM_CON_NAME] )
   next
   _nControlActivo := aScan( acControl, cControl )

////////// Pone la etiqueta en negrita.----------------------------------------
   _HMG_cMacroTemp := _aEtiqueta[_nControlActivo,ABM_LBL_NAME]
   wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).FontBold := .t.

////////// Presenta el mensaje en la barra de estado.--------------------------
   wndABM3EditNuevo.StatusBar.Item( 1 ) := _aControl[_nControlActivo,ABM_CON_DES]

return nil



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3SinFoco()
 * Descripcin: Restaura el estado de las etiquetas y de la barra de estado de la ventana
 *              de edicin de registros al dejar un control de edicin sin foco.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3SinFoco()

////////// Restaura el estado de la etiqueta.----------------------------------
   _HMG_cMacroTemp := _aEtiqueta[_nControlActivo,ABM_LBL_NAME]
   wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).FontBold := .f.

////////// Restaura el texto de la barra de estado.----------------------------
   wndABM3EditNuevo.StatusBar.Item( 1 ) := ""

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3AlEntrar()
 * Descripcin: Cambia al siguiente control de edicin tipo TEXTBOX al pulsar la tecla
 *              ENTER.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3AlEntrar()

////////// Declaracin de variables locales.-----------------------------------
   local lSalida   as logical              // * Tipo de salida.
   local nTipo     as numeric              // * Tipo del control.

////////// Inicializa las variables.-------------------------------------------
   lSalida  := .t.

////////// Restaura el estado de la etiqueta.----------------------------------
   _HMG_cMacroTemp := _aEtiqueta[_nControlActivo,ABM_LBL_NAME]
   wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).FontBold := .f.

////////// Activa el siguiente control editable con evento ON ENTER.-----------
   do while lSalida
      _nControlActivo++
      if _nControlActivo > HMG_LEN( _aControl )
         _nControlActivo := 1
      endif
      nTipo := _aControl[_nControlActivo,ABM_CON_TYPE]
      if nTipo == ABM_TEXTBOXC .or. nTipo == ABM_TEXTBOXN
         if _aEditable[_nControlActivo]
            lSalida := .f.
         endif
      endif
   enddo
   _HMG_cMacroTemp := _aControl[_nControlActivo,ABM_CON_NAME]
   wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).SetFocus

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3EditarGuardar( lNuevo )
 * Descripcin: Aade o guarda el registro en la bdd.
 *  Parmetros: lNuevo          Valor lgico que indica si se est aadiendo un registro
 *                              o editando el actual.
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3EditarGuardar( lNuevo )

////////// Declaracin de variables locales.-----------------------------------
   local i          as numeric             // * Indice de iteracin.
   local xValor                            // * Valor a guardar.
   local lResultado as logical             // * Resultado del bloque del usuario.
   local aValores   as array               // * Valores del registro.

////////// Guarda el registro.-------------------------------------------------
   if _bGuardar == NIL

      // No hay bloque de cdigo del usuario.
      if lNuevo
         (_cArea)->( dbAppend() )
      endif

      if (_cArea)->(rlock())

         for i := 1 to HMG_LEN( _aEstructura )
            _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
            xValor := wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value
*<< begin modif ListBox  /  ComboBox
            IF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE] .and. valtype( aCombos[i] ) = "C"      // it is a ComboBox
                // xValor := wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value
            ELSEIF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE] .and. valtype( aCombos[i] ) = "A"  // it is a ListBox Param
               aItems := aCombos[i]                                               // select the value to be written
               xValor := IIF(0= xValor, "", aItems[xValor] )                        // (after it was selected)
            ELSEIF  ABM_COMBOBOX = _aControl[i,ABM_CON_TYPE]      // it is a ListBox (function)
               cFuncName := "LB_" + _cArea + "_" + _aEstructura[i, DBS_NAME] + "()"
               aItems := &(cFuncName)                                               // select the value to be written
               xValor := IIF(0= xValor, "", aItems[xValor] )                        // (after it was selected)
            ENDIF
*>> end modif ListBox  /  ComboBox
               (_cArea)->( FieldPut( i, xValor ) )
         next
          Unlock

            // Refresca la ventana de visualizacin.
          wndABM3EditNuevo.Release
          ABM3Redibuja( .t. )

      else
         Msgstop ('Record locked by another user')
      endif
   else

      // Hay bloque de cdigo del usuario.
      aValores := {}
      for i := 1 to HMG_LEN( _aControl )
         _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
         xValor := wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value
         aAdd( aValores, xValor )
      next
      lResultado := Eval( _bGuardar, aValores, lNuevo )
      if ValType( lResultado ) != "L"
         lResultado := .t.
      endif

      // Refresca la ventana de visualizacin.
      if lResultado
         wndABM3EditNuevo.Release
         ABM3Redibuja( .t. )
      endif
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Seleccionar()
 * Descripcin: Presenta una ventana para la seleccin de un registro.
 *  Parmetros: Ninguno
 *    Devuelve: [nReg]          Numero de registro seleccionado, o cero si no se ha
 *                              seleccionado ninguno.
****************************************************************************************/
static function ABM3Seleccionar()

////////// Declaracin de variables locales.-----------------------------------
   local lSalida   as logical              // Control de bucle.
   local nReg      as numeric              // Valores del registro
   local nRegistro as numeric              // Nmero de registro.

////////// Inicializacin de variables.----------------------------------------
   lSalida   := .f.
   nReg      := 0
   nRegistro := (_cArea)->( RecNo() )

////////// Se situa en el primer registro.-------------------------------------
   (_cArea)->( dbGoTop() )

////////// Creacin de la ventana de seleccin de registro.--------------------
   define window wndSeleccionar            ;
      AT 0, 0                         ;
      width 500                       ;
      height 300                      ;
      title _HMG_SYSDATA [ 129 ][8]            ;
      modal                           ;
      nosize                          ;
      nosysmenu                       ;
      font "ms sans serif" size 9

      // Define la barra de botones de la ventana de seleccin.
      define toolbar tbSeleccionar buttonsize 90, 32 flat righttext border
         button tbbCancelarSel caption _HMG_SYSDATA [ 128 ][7]                   ;
                     picture "HMG_EDIT_CANCEL"             ;
                     action  {|| lSalida := .f.,               ;
                       nReg    := 0,                 ;
                       wndSeleccionar.Release }
         button tbbAceptarSel  caption _HMG_SYSDATA [ 128 ][8]                                           ;
                     picture "HMG_EDIT_OK"                                         ;
                     action  {|| lSalida := .t.,                                       ;
                       nReg    := wndSeleccionar.brwSeleccionar.Value,       ;
                       wndSeleccionar.Release }
      end toolbar

      // Define la barra de estado de la ventana de seleccin.
      define statusbar font "ms sans serif" size 9
         statusitem _HMG_SYSDATA [ 130 ][7]
      end statusbar

      // Define la tabla de la ventana de seleccin.
      @ 55, 20 browse brwSeleccionar                                          ;
         width 460                                                       ;
         height 190                                                      ;
         headers _aCabeceraTabla                                         ;
         widths _aAnchoTabla                                             ;
         workarea &_cArea                                                ;
         fields _aCampoTabla                                             ;
         value (_cArea)->( RecNo() )                                     ;
         font "arial" size 9                                             ;
         on dblclick {|| lSalida := .t.,                                 ;
               nReg := wndSeleccionar.brwSeleccionar.Value,    ;
               wndSeleccionar.Release }                        ;
         justify _aAlineadoTabla

   end window

////////// Activa la ventana de seleccin de registro.-------------------------
   center window wndSeleccionar
   activate window wndSeleccionar

////////// Restuara el puntero de registro.------------------------------------
   (_cArea)->( dbGoTo( nRegistro ) )

return ( nReg )



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3EditarCopiar()
 * Descripcin: Copia el registro seleccionado en los controles de edicin del nuevo
 *              registro.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3EditarCopiar()

////////// Declaracin de variables locales.-----------------------------------
   local i         as numeric              // Indice de iteracin.
   local nRegistro as numeric              // Puntero de registro.
   local nReg      as numeric              // Numero de registro.

////////// Obtiene el registro a copiar.---------------------------------------
   nReg := ABM3Seleccionar()

////////// Actualiza los controles de edicin.---------------------------------
   if nReg != 0
      nRegistro := (_cArea)->( RecNo() )
      (_cArea)->( dbGoTo( nReg ) )
      for i := 1 to HMG_LEN( _aControl )
         if _aEditable[i]
            _HMG_cMacroTemp := _aControl[i,ABM_CON_NAME]
            wndABM3EditNuevoSplit.&(_HMG_cMacroTemp).Value := ;
               (_cArea)->( FieldGet( i ) )
         endif
      next
      (_cArea)->( dbGoTo( nRegistro ) )
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Borrar()
 * Descripcin: Borra el registro activo.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
function ABM3Borrar()

////////// Declaracin de variables locales.-----------------------------------

////////// Borra el registro si se acepta.-------------------------------------

   if MsgOKCancel( _HMG_SYSDATA [ 130 ][8], _HMG_SYSDATA [ 129 ][16] )
      if (_cArea)->( rlock() )
         (_cArea)->( dbDelete() )
         (_cArea)->( dbCommit() )
         (_cArea)->( dbunlock() )
         if .not. set( _SET_DELETED )
            set deleted on
         endif
         (_cArea)->( dbSkip() )
         if (_cArea)->( eof() )
            (_cArea)->( dbGoBottom() )
         endif
         ABM3Redibuja( .t. )
      else
         Msgstop( _HMG_SYSDATA [ 130 ] [41] , _cTitulo )
      endif
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Buscar()
 * Descripcin: Busca un registro por la clave del indice activo.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3Buscar()

////////// Declaracin de variables locales.-----------------------------------
   local nControl   as numeric             // Numero del control.
   local lSalida    as logical             // Tipo de salida de la ventana.
   local xValor                            // Valor de busqueda.
   local cMascara   as character           // Mascara de edicin del control.
   local lResultado as logical             // Resultado de la busqueda.
   local nRegistro  as numeric             // Numero de registro.

////////// Inicializacin de variables.----------------------------------------
   nControl := _aIndiceCampo[_nIndiceActivo]

////////// Comprueba si se ha pasado una accin del usuario.--------------------
   if _bBuscar != NIL
      // msgInfo( "ON FIND" )
      Eval( _bBuscar )
      ABM3Redibuja( .t. )
      return NIL
   endif

////////// Comprueba si hay un indice activo.----------------------------------
   if _nIndiceActivo == 1
      msgExclamation( _HMG_SYSDATA [ 130 ][9], _cTitulo )
      return NIL
   endif

////////// Comprueba que el campo indice no es del tipo memo o logico.---------
   if _aEstructura[nControl,DBS_TYPE] == "L" .or. _aEstructura[nControl,DBS_TYPE] == "M"
      msgExclamation( _HMG_SYSDATA [ 130 ][10], _cTitulo )
      return nil
   endif

////////// Crea la ventana de busqueda.----------------------------------------
   define window wndABMBuscar              ;
      AT 0, 0                         ;
      width 500                       ;
      height 170                      ;
      title _HMG_SYSDATA [ 129 ][9]            ;
      modal                           ;
      nosize                          ;
      nosysmenu                       ;
      font "ms sans serif" size 9

      // Define la barra de botones de la ventana de busqueda.
      define toolbar tbBuscar buttonsize 90, 32 flat righttext border
         button tbbCancelarBus caption _HMG_SYSDATA [ 128 ][7]                           ;
                     picture "HMG_EDIT_CANCEL"                     ;
                     action  {|| lSalida := .f.,                       ;
                       xValor := wndABMBuscar.conBuscar.Value,  ;
                       wndABMBuscar.Release }
         button tbbAceptarBus  caption _HMG_SYSDATA [ 128 ][8]                                ;
                     picture "HMG_EDIT_OK"                         ;
                     action  {|| lSalida := .t.,                       ;
                       xValor := wndABMBuscar.conBuscar.Value,  ;
                       wndABMBuscar.Release }
      end toolbar

      // Define la barra de estado de la ventana de busqueda.
      define statusbar font "ms sans serif" size 9
         statusitem ""
      end statusbar
   end window

////////// Crea los controles de la ventana de busqueda.-----------------------
   // Frame.
   @ 45, 10 frame frmBuscar                        ;
      of wndABMBuscar                         ;
      caption ""                              ;
      width wndABMBuscar.Width - 25           ;
      height wndABMBuscar.Height - 100

   // Etiqueta.
   @ 60, 20 label lblBuscar                                ;
      of wndABMBuscar                                 ;
      value _aNombreCampo[nControl]                   ;
      width _aEtiqueta[nControl,ABM_LBL_WIDTH]        ;
      height _aEtiqueta[nControl,ABM_LBL_HEIGHT]      ;
      font "ms sans serif" size 9

   // Tipo de dato a buscar.
   do case

      // Carcter.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_TEXTBOXC
         @ 75, 20  textbox conBuscar                             ;
            of wndABMBuscar                                    ;
            value ""                                        ;
            height _aControl[nControl,ABM_CON_HEIGHT]       ;
            width _aControl[nControl,ABM_CON_WIDTH]         ;
            font "arial" size 9                             ;
            maxlength _aEstructura[nControl,DBS_LEN]

      // Fecha.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_DATEPICKER
         @ 75, 20 datepicker conBuscar                           ;
            of wndABMBuscar                                    ;
            value Date()                                    ;
            height _aControl[nControl,ABM_CON_HEIGHT]       ;
            width _aControl[nControl,ABM_CON_WIDTH] + 25    ;
            font "arial" size 9

      // Numerico.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_TEXTBOXN
         if ( _aEstructura[nControl,DBS_DEC] == 0 )

            // Sin decimales.
            @ 75, 20 textbox conBuscar                              ;
               of wndABMBuscar                                    ;
               value ""                                        ;
               height _aControl[nControl,ABM_CON_HEIGHT]       ;
               width _aControl[nControl,ABM_CON_WIDTH]         ;
               numeric                                         ;
               font "arial" size 9                             ;
               maxlength _aEstructura[nControl,DBS_LEN]
         else

            // Con decimales.
            cMascara := ""
            cMascara := REPLICATE( "9", _aEstructura[nControl,DBS_LEN] - ;
                         ( _aEstructura[nControl,DBS_DEC] + 1 ) )
            cMascara += "."
            cMascara += REPLICATE( "9", _aEstructura[nControl,DBS_DEC] )
            @ 75, 20 textbox conBuscar                              ;
               of wndABMBuscar                                    ;
               value ""                                        ;
               height _aControl[nControl,ABM_CON_HEIGHT]       ;
               width _aControl[nControl,ABM_CON_WIDTH]         ;
               numeric                                         ;
               inputmask cMascara
         endif
   endcase

////////// Actualiza la barra de estado.---------------------------------------
   wndABMBuscar.StatusBar.Item( 1 ) := _aControl[nControl,ABM_CON_DES]

////////// Comprueba el tamao del control de edicin del dato a buscar.-------
   if wndABMBuscar.conBuscar.Width > wndABM3Edit.Width - 45
      wndABMBuscar.conBuscar.Width := wndABM3Edit.Width - 45
   endif

////////// Activa la ventana de busqueda.--------------------------------------
   center window wndABMBuscar
   activate window wndABMBuscar

////////// Busca el registro.--------------------------------------------------
   if lSalida
      nRegistro := (_cArea)->( RecNo() )
      lResultado := (_cArea)->( dbSeek( xValor ) )
      if !lResultado
         msgExclamation( _HMG_SYSDATA [ 130 ][11], _cTitulo )
         (_cArea)->( dbGoTo( nRegistro ) )
      else
         ABM3Redibuja( .t. )
      endif
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Filtro()
 * Descripcin: Filtra la base de datos.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3ActivarFiltro()

////////// Declaracin de variables locales.-----------------------------------
   local aCompara   as array               // Comparaciones.
   local aCampos    as array               // Nombre de los campos.

////////// Comprueba que no hay ningun filtro activo.--------------------------
   if _cFiltro != ""
      MsgInfo( _HMG_SYSDATA [ 130 ][34], '' )
   endif

////////// Inicializacin de variables.----------------------------------------
   aCampos    := _aNombreCampo
   aCompara   := { _HMG_SYSDATA [ 129 ][27],;
         _HMG_SYSDATA [ 129 ][28],;
         _HMG_SYSDATA [ 129 ][29],;
         _HMG_SYSDATA [ 129 ][30],;
         _HMG_SYSDATA [ 129 ][31],;
         _HMG_SYSDATA [ 129 ][32] }


////////// Crea la ventana de filtrado.----------------------------------------
   define window wndABM3Filtro                     ;
      AT 0, 0                                 ;
      width 400                               ;
      height 325                              ;
      title _HMG_SYSDATA [ 129 ][21]            ;
      modal                                   ;
      nosize                                  ;
      nosysmenu                               ;
      on init {|| ABM3ControlFiltro() }       ;
      font "ms sans serif" size 9

      // Define la barra de botones de la ventana de filtrado.
      define toolbar tbBuscar buttonsize 90, 32 flat righttext border
         button tbbCancelarFil caption _HMG_SYSDATA [ 128 ][7]           ;
                     picture "HMG_EDIT_CANCEL"     ;
                     action  {|| wndABM3Filtro.Release,;
                       ABM3Redibuja( .f. ) }
         button tbbAceptarFil  caption _HMG_SYSDATA [ 128 ][8]           ;
                     picture "HMG_EDIT_OK"         ;
                     action  {|| ABM3EstableceFiltro() }
      end toolbar

      // Define la barra de estado de la ventana de filtrado.
      define statusbar font "ms sans serif" size 9
         statusitem ""
      end statusbar
   end window

////////// Controles de la ventana de filtrado.
   // Frame.
   @ 45, 10 frame frmFiltro                        ;
      of wndABM3Filtro                        ;
      caption ""                              ;
      width wndABM3Filtro.Width - 25          ;
      height wndABM3Filtro.Height - 100
   @ 65, 20 label lblCampos                ;
      of wndABM3Filtro                ;
      value _HMG_SYSDATA [ 129 ][22]        ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 65, 220 label lblCompara              ;
      of wndABM3Filtro                ;
      value _HMG_SYSDATA [ 129 ][23]    ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 200, 20 label lblValor                ;
      of wndABM3Filtro                ;
      value _HMG_SYSDATA [ 129 ][24]        ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 85, 20 listbox lbxCampos                      ;
      of wndABM3Filtro                        ;
      width 140                               ;
      height 100                              ;
      items aCampos                           ;
      value 1                                 ;
      font "Arial" size 9                     ;
      on change {|| ABM3ControlFiltro() }     ;
      on gotfocus wndABM3Filtro.StatusBar.Item(1) := _HMG_SYSDATA [ 129 ][25] ;
      on lostfocus wndABM3Filtro.StatusBar.Item(1) := ""
   @ 85, 220 listbox lbxCompara                    ;
      of wndABM3Filtro                        ;
      width 140                               ;
      height 100                              ;
      items aCompara                          ;
      value 1                                 ;
      font "Arial" size 9                     ;
      on gotfocus wndABM3Filtro.StatusBar.Item(1) := _HMG_SYSDATA [ 129 ][26] ;
      on lostfocus wndABM3Filtro.StatusBar.Item(1) := ""
   @ 220, 20 textbox conValor              ;
      of wndABM3Filtro                ;
      value ""                        ;
      height 25                       ;
      width 160                       ;
      font "arial" size 9             ;
      maxlength 16

////////// Activa la ventana.
   center window wndABM3Filtro
   activate window wndABM3Filtro


return NIL


/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3ControlFiltro()
 * Descripcin: Comprueba que el filtro se puede aplicar.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3ControlFiltro()

////////// Declaracin de variables locales.-----------------------------------
   local nControl as numeric
   local cMascara as character
   local cMensaje as character

////////// Inicializa las variables.
   nControl := wndABM3Filtro.lbxCampos.Value

///////// Comprueba que se puede crear el control.-----------------------------
   if _aEstructura[nControl,DBS_TYPE] == "M"
      msgExclamation( _HMG_SYSDATA [ 130 ][35], _cTitulo )
      return NIL
   endif
   if nControl == 0
      msgExclamation( _HMG_SYSDATA [ 130 ][36], _cTitulo )
      return NIL
   endif

///////// Crea el nuevo control.-----------------------------------------------
   wndABM3Filtro.conValor.Release
   cMensaje := _aControl[nControl,ABM_CON_DES]
   do case

      // Carcter.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_TEXTBOXC
         @ 226, 20  textbox conValor                                     ;
            of wndABM3Filtro                                        ;
            value ""                                                ;
            height _aControl[nControl,ABM_CON_HEIGHT]               ;
            width _aControl[nControl,ABM_CON_WIDTH]                 ;
            font "arial" size 9                                     ;
            maxlength _aEstructura[nControl,DBS_LEN]                ;
            on gotfocus wndABM3Filtro.StatusBar.Item( 1 ) :=        ;
                   cMensaje                                    ;
            on lostfocus wndABM3Filtro.StatusBar.Item( 1 ) := ""

      // Fecha.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_DATEPICKER
         @ 226, 20 datepicker conValor                                   ;
            of wndABM3Filtro                                        ;
            value Date()                                            ;
            height _aControl[nControl,ABM_CON_HEIGHT]               ;
            width _aControl[nControl,ABM_CON_WIDTH] + 25            ;
            font "arial" size 9                                     ;
            on gotfocus wndABM3Filtro.StatusBar.Item( 1 ) :=        ;
                   cMensaje                                    ;
            on lostfocus wndABM3Filtro.StatusBar.Item( 1 ) := ""

      // Numerico.
      case _aControl[nControl,ABM_CON_TYPE] == ABM_TEXTBOXN
         if ( _aEstructura[nControl,DBS_DEC] == 0 )

            // Sin decimales.
            @ 226, 20 textbox conValor                                      ;
               of wndABM3Filtro                                        ;
               value ""                                                ;
               height _aControl[nControl,ABM_CON_HEIGHT]               ;
               width _aControl[nControl,ABM_CON_WIDTH]                 ;
               numeric                                                 ;
               font "arial" size 9                                     ;
               maxlength _aEstructura[nControl,DBS_LEN]                ;
               on gotfocus wndABM3Filtro.StatusBar.Item( 1 ) :=        ;
                      cMensaje                                    ;
               on lostfocus wndABM3Filtro.StatusBar.Item( 1 ) := ""

         else

            // Con decimales.
            cMascara := ""
            cMascara := REPLICATE( "9", _aEstructura[nControl,DBS_LEN] - ;
                         ( _aEstructura[nControl,DBS_DEC] + 1 ) )
            cMascara += "."
            cMascara += REPLICATE( "9", _aEstructura[nControl,DBS_DEC] )
            @ 226, 20 textbox conValor                                      ;
               of wndABM3Filtro                                        ;
               value ""                                                ;
               height _aControl[nControl,ABM_CON_HEIGHT]               ;
               width _aControl[nControl,ABM_CON_WIDTH]                 ;
               numeric                                                 ;
               inputmask cMascara                                      ;
               on gotfocus wndABM3Filtro.StatusBar.Item( 1 ) :=        ;
                      cMensaje                                    ;
               on lostfocus wndABM3Filtro.StatusBar.Item( 1 ) := ""
         endif

      // Logico
      case _aControl[nControl,ABM_CON_TYPE] == ABM_CHECKBOX
         @ 226, 20 checkbox conValor                                     ;
            of wndABM3Filtro                                        ;
            caption ""                                              ;
            height _aControl[nControl,ABM_CON_HEIGHT]               ;
            width _aControl[nControl,ABM_CON_WIDTH]                 ;
            value .f.                                               ;
            on gotfocus wndABM3Filtro.StatusBar.Item( 1 ) :=        ;
                   cMensaje                                    ;
            on lostfocus wndABM3Filtro.StatusBar.Item( 1 ) := ""

   endcase

////////// Actualiza el valor de la etiqueta.----------------------------------
   wndABM3Filtro.lblValor.Value := _aNombreCampo[nControl]

////////// Actualiza la barra de estado.---------------------------------------
   //wndABM3Filtro.StatusBar.Item( 1 ) := _aControl[nControl,ABM_CON_DES]

////////// Comprueba el tamao del control de edicin del dato a buscar.-------
   if wndABM3Filtro.conValor.Width > wndABM3Filtro.Width - 45
      wndABM3Filtro.conValor.Width := wndABM3Filtro.Width - 45
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3EstableceFiltro()
 * Descripcin: Establece el filtro seleccionado.
 *  Parmetros: Ninguno.
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3EstableceFiltro()

////////// Declaracin de variables locales.-----------------------------------
   local aOperador  as array
   local nCampo     as numeric
   local nCompara   as numeric
   local cValor     as character

////////// Inicializacin de variables.----------------------------------------
   nCompara  := wndABM3Filtro.lbxCompara.Value
   nCampo    := wndABM3Filtro.lbxCampos.Value
   cValor    := HB_ValToStr( wndABM3Filtro.conValor.Value )
   aOperador := { "=", "<>", ">", "<", ">=", "<=" }

////////// Comprueba que se puede filtrar.-------------------------------------
   if nCompara == 0
      msgExclamation( _HMG_SYSDATA [ 130 ][37], _cTitulo )
      return NIL
   endif
   if nCampo == 0
      msgExclamation( _HMG_SYSDATA [ 130 ][36], _cTitulo )
      return NIL
   endif
   if cValor == ""
      msgExclamation( _HMG_SYSDATA [ 130 ][38], _cTitulo )
      return NIL
   endif
   if _aEstructura[nCampo,DBS_TYPE] == "M"
      msgExclamation( _HMG_SYSDATA [ 130 ][35], _cTitulo )
      return NIL
   endif

////////// Establece el filtro.------------------------------------------------
   do case
      case _aEstructura[nCampo,DBS_TYPE] == "C"
          _cFiltro := "HMG_UPPER(" + _cArea + "->" + ;
            _aEstructura[nCampo,DBS_NAME] + ")"+ ;
            aOperador[nCompara]
          _cFiltro += "'" + HMG_UPPER( ALLTRIM( cValor ) ) + "'"

      case _aEstructura[nCampo,DBS_TYPE] == "N"
          _cFiltro := _cArea + "->" + ;
            _aEstructura[nCampo,DBS_NAME] + ;
            aOperador[nCompara]
          _cFiltro += ALLTRIM( cValor )

      case _aEstructura[nCampo,DBS_TYPE] == "D"
          _cFiltro := _cArea + "->" + ;
            _aEstructura[nCampo,DBS_NAME] + ;
            aOperador[nCompara]
          _cFiltro += "CToD(" + "'" + cValor + "')"

      case _aEstructura[nCampo,DBS_TYPE] == "L"
          _cFiltro := _cArea + "->" + ;
            _aEstructura[nCampo,DBS_NAME] + ;
            aOperador[nCompara]
          _cFiltro += cValor
   endcase
   (_cArea)->( dbSetFilter( {|| &_cFiltro}, _cFiltro ) )
   _lFiltro := .t.
   wndABM3Filtro.Release
   ABM3Redibuja( .t. )

return NIL


/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin:ABM3DesactivarFiltro()
 * Descripcin:Desactiva el filtro seleccionado
 *  Parmetros:Ninguno.
 *    Devuelve:NIL
****************************************************************************************/
static function ABM3DesactivarFiltro()

////////// Desactiva el filtro si procede.
   if !_lFiltro
      msgExclamation( _HMG_SYSDATA [ 130 ][39], _cTitulo )
      ABM3Redibuja( .f. )
      return NIL
   endif
   if msgYesNo( _HMG_SYSDATA [ 130 ][40], _cTitulo )
      (_cArea)->( dbSetFilter( {|| NIL }, "" ) )
      _lFiltro := .f.
      _cFiltro := ""
      ABM3Redibuja( .t. )
   endif

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Imprimir()
 * Descripcin: Presenta la ventana de recogida de datos para la definicin del listado.
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3Imprimir()

////////// Declaracin de variables locales.-----------------------------------
   local aCampoBase    as array            // Campos de la bdd.
   local aCampoListado as array            // Campos del listado.
   local nRegistro     as numeric          // Numero de registro actual.
   local nCampo        as numeric          // Numero de campo.
   local cRegistro1    as character        // Valor del registro inicial.
   local cRegistro2    as character        // Valor del registro final.
   local aImpresoras   as array            // Impresoras disponibles.
   local NIMPLEN
   private hbprn

////////// Comprueba si se ha pasado la clausula ON PRINT.---------------------
   IF _bImprimir != NIL
      // msgInfo( "ON PRINT" )
      Eval( _bImprimir )
      ABM3Redibuja( .T. )
      RETURN NIL
   ENDIF

////////// Obtiene las impresoras disponibles.---------------------------------
   aImpresoras := {}
   aImpresoras := aPrinters()
   if ValType( nImpLen ) # 'N'
      nImpLen := HMG_LEN( aImpresoras )
   endif
   aSize( aImpresoras, nImpLen )

////////// Comprueba que hay un indice activo.---------------------------------
   if _nIndiceActivo == 1
      msgExclamation( _HMG_SYSDATA [ 130 ][9], _cTitulo )
      return NIL
   endif

////////// Inicializacin de variables.----------------------------------------
   aCampoListado := {}
   aCampoBase    := _aNombreCampo
   nRegistro     := (_cArea)->( RecNo() )

   // Registro inicial y final.
   nCampo := _aIndiceCampo[_nIndiceActivo]
   ( _cArea)->( dbGoTop() )
   cRegistro1 := HB_ValToStr( (_cArea)->( FieldGet( nCampo ) ) )
   ( _cArea)->( dbGoBottom() )
   cRegistro2 := HB_ValToStr( (_cArea)->( FieldGet( nCampo ) ) )
   (_cArea)->( dbGoTo( nRegistro ) )

////////// Definicin de la ventana de formato de listado.---------------------
   define window wndABM3Listado            ;
      AT 0, 0                         ;
      width 390                       ;
      height 365                      ;
      title _HMG_SYSDATA [ 129 ][10]   ;
      icon "HMG_EDIT_PRINT"       ;
      modal                           ;
      nosize                          ;
      nosysmenu                       ;
      font "ms sans serif" size 9

      // Define la barra de botones de la ventana de formato de listado.
      define toolbar tbListado buttonsize 90, 32 flat righttext border
         button tbbCancelarLis caption _HMG_SYSDATA [ 128 ][7]                   ;
                     picture "HMG_EDIT_CANCEL"             ;
                     action  wndABM3Listado.Release
         button tbbAceptarLis  caption _HMG_SYSDATA [ 128 ][8]                   ;
                     picture "HMG_EDIT_OK"                 ;
                     action  ABM3Listado( aImpresoras )

      end toolbar

      // Define la barra de estado de la ventana de formato de listado.
      define statusbar font "ms sans serif" size 9
         statusitem ""
      end statusbar
   end window

////////// Define los controles de edicin de la ventana de formato de listado.-
   // Frame.
   @ 45, 10 frame frmListado                       ;
      of wndABM3Listado                       ;
      caption ""                              ;
      width wndABM3Listado.Width - 25         ;
      height wndABM3Listado.Height - 100

   // Label
   @ 65, 20 label lblCampoBase             ;
      of wndABM3Listado               ;
      value _HMG_SYSDATA [ 129 ][11]       ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 65, 220 label lblCampoListado         ;
      of wndABM3Listado               ;
      value _HMG_SYSDATA [ 129 ][12]           ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 200, 20 label lblImpresoras           ;
      of wndABM3Listado               ;
      value _HMG_SYSDATA [ 129 ][13]   ;
      width 140                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 200, 170 label lblInicial             ;
      of wndABM3Listado               ;
      value _HMG_SYSDATA [ 129 ][14]           ;
      width 160                       ;
      height 25                       ;
      font "ms sans serif" size 9
   @ 255, 170 label lblFinal               ;
      of wndABM3Listado               ;
      value _HMG_SYSDATA [ 129 ][15]           ;
      width 160                       ;
      height 25                       ;
      font "ms sans serif" size 9

   // Listbox.
   @ 85, 20 listbox lbxCampoBase                                           ;
      of wndABM3Listado                                               ;
      width 140                                                       ;
      height 100                                                      ;
      items aCampoBase                                                ;
      value 1                                                         ;
      font "Arial" size 9                                             ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][12] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""
   @ 85, 220 listbox lbxCampoListado                                       ;
      of wndABM3Listado                                               ;
      width 140                                                       ;
      height 100                                                      ;
      items aCampoListado                                             ;
      value 1                                                         ;
      font "Arial" size 9                                             ;
      on gotFocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][13];
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""

   // ComboBox.
   @ 220, 20 combobox cbxImpresoras                                        ;
      of wndABM3Listado                                               ;
      items aImpresoras                                               ;
      value 1                                                         ;
      width 140                                                       ;
      font "Arial" size 9                                             ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][14] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""

   // PicButton.
   @ 90, 170 button btnMas                                                 ;
      of wndABM3Listado                                               ;
      picture "HMG_EDIT_ADD"                                      ;
      action ABM3DefinirColumnas( ABM_LIS_ADD )                       ;
      width 40                                                        ;
      height 40                                                       ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][15] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""
   @ 140, 170 button btnMenos                                              ;
      of wndABM3Listado                                               ;
      picture "HMG_EDIT_DEL"                                      ;
      action ABM3DefinirColumnas( ABM_LIS_DEL )                       ;
      width 40                                                        ;
      height 40                                                       ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][16] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""
   @ 220, 170 button btnSet1                                               ;
      of wndABM3Listado                                               ;
      picture "HMG_EDIT_SET"                                      ;
      action ABM3DefinirRegistro( ABM_LIS_SET1 )                      ;
      width 25                                                        ;
      height 25                                                       ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][17] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""
   @ 275, 170 button btnSet2                                               ;
      of wndABM3Listado                                               ;
      picture "HMG_EDIT_SET"                                      ;
      action ABM3DefinirRegistro( ABM_LIS_SET2 )                      ;
      width 25                                                        ;
      height 25                                                       ;
      on gotfocus wndABM3Listado.StatusBar.Item( 1 ) := _HMG_SYSDATA [ 130 ][18] ;
      on lostfocus wndABM3Listado.StatusBar.Item( 1 ) := ""

   // CheckBox.

   @ 275, 20 checkbox chkPrevio            ;
      of wndABM3Listado               ;
      caption _HMG_SYSDATA [ 129 ][17]      ;
      width 140                       ;
      height 25                       ;
      value .t.                       ;
      font "ms sans serif" size 9

   // Editbox.
   @ 220, 196 textbox txtRegistro1         ;
      of wndABM3Listado               ;
      value cRegistro1                ;
      height 25                       ;
      width 160                       ;
      font "arial" size 9             ;
      maxlength 16
   @ 275, 196 textbox txtRegistro2         ;
      of wndABM3Listado               ;
      value cRegistro2                ;
      height 25                       ;
      width 160                       ;
      font "arial" size 9             ;
      maxlength 16

////////// Estado de los controles.--------------------------------------------
   wndABM3Listado.txtRegistro1.Enabled := .f.
   wndABM3Listado.txtRegistro2.Enabled := .f.

////////// Comrprueba que la seleccin de registros es posible.----------------
   nCampo := _aIndiceCampo[_nIndiceActivo]
   if _aEstructura[nCampo,DBS_TYPE] == "L" .or. _aEstructura[nCampo,DBS_TYPE] == "M"
      wndABM3Listado.btnSet1.Enabled := .f.
      wndABM3Listado.btnSet2.Enabled := .f.
   endif

////////// Activacin de la ventana de formato de listado.---------------------
   center window wndABM3Listado
   activate window wndABM3Listado

////////// Restaura.-----------------------------------------------------------
   (_cArea)->( dbGoTo( nRegistro ) )
   ABM3Redibuja( .f. )

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3DefinirRegistro( nAccion )
 * Descripcin:
 *  Parmetros: nAccion
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3DefinirRegistro( nAccion )

////////// Declaracin de variables locales.-----------------------------------
   local nRegistro as character            // * Puntero de registros.
   local nReg      as character            // * Registro seleccionado.
   local cValor    as character            // * Valor del registro seleccionado.
   local nCampo    as numeric              // * Numero del campo indice.

////////// Inicializa las variables.-------------------------------------------
   nRegistro := (_cArea)->( RecNo() )
   cValor    := ""

////////// Selecciona el registro.---------------------------------------------
   nReg := ABM3Seleccionar()
   if nReg == 0
      (_cArea)->( dbGoTo( nRegistro ) )
      return NIL
   else
      (_cArea)->( dbGoTo( nReg ) )
      nCampo := _aIndiceCampo[_nIndiceActivo]
      cValor := HB_ValToStr( (_cArea)->( FieldGet( nCampo ) ) )
   endif

////////// Actualiza segn la accin.------------------------------------------
   do case
      case nAccion == ABM_LIS_SET1
         wndABM3Listado.txtRegistro1.Value := cValor
      case nAccion == ABM_LIS_SET2
         wndABM3Listado.txtRegistro2.Value := cValor
   endcase

////////// Restaura el registro.
   (_cArea)->( dbGoTo( nRegistro ) )

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3DefinirColumnas( nAccion )
 * Descripcin: Controla el contenido de las listas al pulsar los botones de aadir y
 *              eliminar campos del listado.
 *  Parmetros: [nAccion]       Numerico. Indica el tipo de accion realizado.
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3DefinirColumnas( nAccion )

////////// Declaracin de variables locales.-----------------------------------
   local aCampoBase    as array            // * Campos de la bbd.
   local aCampoListado as array            // * Campos del listado.
   local i             as numeric          // * Indice de iteracin.
   local nItem         as numeric          // * Numero del item seleccionado.
   local cvalor

////////// Inicializacin de variables.----------------------------------------
   aCampoBase  := {}
   aCampoListado := {}
   for i := 1 to wndABM3Listado.lbxCampoBase.ItemCount
      aAdd( aCampoBase, wndABM3Listado.lbxCampoBase.Item( i ) )
   next
   for i := 1 to wndABM3Listado.lbxCampoListado.ItemCount
      aAdd( aCampoListado, wndABM3Listado.lbxCampoListado.Item( i ) )
   next

////////// Ejecuta segn la accin.--------------------------------------------
   do case
      case nAccion == ABM_LIS_ADD

         // Obtiene la columna a aadir.
         nItem := wndABM3Listado.lbxCampoBase.Value
         cValor := wndABM3Listado.lbxCampoBase.Item( nItem )

         // Actualiza los datos de los campos de la base.
         if HMG_LEN( aCampoBase ) == 0
            msgExclamation( _HMG_SYSDATA [ 130 ][23], _cTitulo )
            return NIL
         else
            wndABM3Listado.lbxCampoBase.DeleteAllItems
            for i := 1 to HMG_LEN( aCampoBase )
               if i != nItem
                  wndABM3Listado.lbxCampoBase.AddItem( aCampoBase[i] )
               endif
            next
            nItem := iif( nItem > 1, nItem--, 1 )
            wndABM3Listado.lbxCampoBase.Value := nItem
         endif

         // Actualiza los datos de los campos del listado.
         if Empty( cValor )
            msgExclamation( _HMG_SYSDATA [ 130 ][23], _cTitulo )
            return NIL
         else
            wndABM3Listado.lbxCampoListado.AddItem( cValor )
            wndABM3Listado.lbxCampoListado.Value := ;
               wndABM3Listado.lbxCampoListado.ItemCount
         endif
      case nAccion == ABM_LIS_DEL

         // Obtiene la columna a quitar.
         nItem := wndABM3Listado.lbxCampoListado.Value
         cValor := wndABM3Listado.lbxCampoListado.Item( nItem )

         // Actualiza los datos de los campos del listado.
         if HMG_LEN( aCampoListado ) == 0
            msgExclamation( _HMG_SYSDATA [ 130 ][23], _cTitulo )
            return NIL
         else
            wndABM3Listado.lbxCampoListado.DeleteAllItems
            for i := 1 to HMG_LEN( aCampoListado )
               if i != nItem
                  wndABM3Listado.lbxCampoListado.AddItem( aCampoListado[i] )
               endif
            next
            nItem := iif( nItem > 1, nItem--, 1 )
            wndABM3Listado.lbxCampoListado.Value := ;
               wndABM3Listado.lbxCampoListado.ItemCount
         endif

         // Actualiza los datos de los campos de la base.
         if Empty( cValor )
            msgExclamation( _HMG_SYSDATA [ 130 ][23], _cTitulo )
            return NIL
         else
            wndABM3Listado.lbxCampoBase.DeleteAllItems
            for i := 1 to HMG_LEN( _aNombreCampo )
               if aScan( aCampoBase, _aNombreCampo[i] ) != 0
                  wndABM3Listado.lbxCampoBase.AddItem( _aNombreCampo[i] )
               endif
               if _aNombreCampo[i] == cValor
                  wndABM3Listado.lbxCampoBase.AddItem( _aNombreCampo[i] )
               endif
            next
            wndABM3Listado.lbxCampoBase.Value := 1
         endif
   endcase

return NIL



/****************************************************************************************
 *  Aplicacin: Comando EDIT para HMG
 *       Autor: Cristbal Moll [cemese@terra.es]
 *     Funcin: ABM3Listado()
 * Descripcin: Imprime la selecciona realizada por ABM3Imprimir()
 *  Parmetros: Ninguno
 *    Devuelve: NIL
****************************************************************************************/
static function ABM3Listado( aImpresoras )

////////// Declaracin de variables locales.-----------------------------------
   local i             as numeric          // * Indice de iteracin.
   local cCampo        as character        // * Nombre del campo indice.
   local aCampo        as array            // * Nombres de los campos.
   local nCampo        as numeric          // * Numero del campo actual.
   local nPosicion     as numeric          // * Posicin del campo.
   local aNumeroCampo  as array            // * Numeros de los campos.
   local aAncho        as array            // * Anchos de las columnas.
   local nAncho        as array            // * Ancho de las columna actual.
   local lPrevio       as logical          // * Previsualizar.
//        local lVistas       as logical          // * Vistas en miniatura.
   local nImpresora    as numeric          // * Numero de la impresora.
   local cImpresora    as character        // * Nombre de la impresora.
   local lOrientacion  as logical          // * Orientacin de la pgina.
   local lSalida       as logical          // * Control de bucle.
   local lCabecera     as logical          // * Imprimir cabecera?.
   local nFila         as numeric          // * Numero de la fila.
   local nColumna      as numeric          // * Numero de la columna.
   local nPagina       as numeric          // * Numero de pgina.
   local nPaginas      as numeric          // * Pginas totales.
   local cPie          as character        // * Texto del pie de pgina.
   local nPrimero      as numeric          // * Numero del primer registro a imprimir.
   local nUltimo       as numeric          // * Numero del ultimo registro a imprimir.
   local nTotales      as numeric          // * Registros totales a imprimir.
   local nRegistro     as numeric          // * Numero del registro actual.
   local cRegistro1    as character        // * Valor del registro inicial.
   local cRegistro2    as character        // * Valor del registro final.
   local xRegistro1                        // * Valor de comparacin.
   local xRegistro2                        // * Valor de comparacin.
   local lSuccess
   local HBPRNMAXCOL := 100
   LOCAL RF    := 4
   LOCAL CF    := 2

////////// Inicializacin de variables.----------------------------------------
   // Previsualizar.
   lPrevio := wndABM3Listado.chkPrevio.Value

   // Nombre de la impresora.
   nImpresora := wndABM3Listado.cbxImpresoras.Value
   if nImpresora == 0
      msgExclamation( _HMG_SYSDATA [ 130 ][32], '' )
   else
      cImpresora := aImpresoras[nImpresora]
   endif

   // Nombre del campo.
   aCampo := {}
   for i := 1 to wndABM3Listado.lbxCampoListado.ItemCount
      cCampo := wndABM3Listado.lbxCampoListado.Item( i )
      aAdd( aCampo, cCampo )
   next
   if HMG_LEN( aCampo ) == 0
      msgExclamation( _HMG_SYSDATA [ 130 ][23], _cTitulo )
      return NIL
   endif

   // Nmero del campo.
   aNumeroCampo := {}
   for i := 1 to HMG_LEN( aCampo )
      nPosicion := aScan( _aNombreCampo, aCampo[i] )
      aAdd( aNumeroCampo, nPosicion )
   next

////////// Obtiene el ancho de impresin.--------------------------------------
   aAncho := {}
   nAncho := 0
   for i := 1 to HMG_LEN( aNumeroCampo )
      nCampo := aNumeroCampo[i]
      do case
         case _aEstructura[nCampo,DBS_TYPE] == "D"
            nAncho := 9
         case _aEstructura[nCampo,DBS_TYPE] == "M"
            nAncho := 20
         otherwise
            nAncho := _aEstructura[nCampo,DBS_LEN]
      endcase
      nAncho := iif( HMG_LEN( _aNombreCampo[nCampo] ) > nAncho ,  ;
                HMG_LEN( _aNombreCampo[nCampo] ),            ;
                nAncho )
      aAdd( aAncho, 1 + nAncho )
   next

////////// Comprueba el ancho de impresin.------------------------------------
   nAncho := 0
   for i := 1 to HMG_LEN( aAncho )
      nAncho += aAncho[i]
   next
   if nAncho > 164
      MsgExclamation( _HMG_SYSDATA [ 130 ][24], _cTitulo )
      return NIL
   else
      if nAncho > 109                 // Horizontal.
         lOrientacion := .t.
      else                            // Vertical.
         lOrientacion := .f.
      endif
   endif

////////// Valores de inicio y fin de listado.---------------------------------
   nRegistro  := (_cArea)->( RecNo() )
   cRegistro1 := wndABM3Listado.txtRegistro1.Value
   cRegistro2 := wndABM3Listado.txtRegistro2.Value
   do case
      case _aEstructura[_aIndiceCampo[_nIndiceActivo],DBS_TYPE] == "C"
         xRegistro1 := cRegistro1
         xRegistro2 := cRegistro2
      case _aEstructura[_aIndiceCampo[_nIndiceActivo],DBS_TYPE] == "N"
         xRegistro1 := Val( cRegistro1 )
         xRegistro2 := Val( cRegistro2 )
      case _aEstructura[_aIndiceCampo[_nIndiceActivo],DBS_TYPE] == "D"
         xRegistro1 := CToD( cRegistro1 )
         xRegistro2 := CToD( cRegistro2 )
      case _aEstructura[_aIndiceCampo[_nIndiceActivo],DBS_TYPE] == "L"
         xRegistro1 := iif( cRegistro1 == ".t.", .t., .f. )
         xRegistro2 := iif( cRegistro2 == ".t.", .t., .f. )
   endcase
   (_cArea)->( dbSeek( xRegistro2 ) )
   nUltimo := (_cArea)->( RecNo() )
   (_cArea)->( dbSeek( xRegistro1 ) )
   nPrimero := (_cArea)->( RecNo() )

////////// Obtiene el nmero de pginas.---------------------------------------
   nTotales := 0
   do while (_cArea)->( RecNo() ) != nUltimo .or. (_cArea)->( Eof() )
      nTotales++
      (_cArea)->( dbSkip( 1 ) )
   enddo
   if lOrientacion
      if Mod( nTotales, 33 ) == 0
         nPaginas := Int( nTotales / 33 )
      else
         nPaginas := Int( nTotales / 33 ) + 1
      endif
   else
      if Mod( nTotales, 55 ) == 0
         nPaginas := Int( nTotales / 55 )
      else
         nPaginas := Int( nTotales / 55 ) + 1
      endif
   endif
   (_cArea)->( dbGoTo( nPrimero ) )

////////// Inicializa el listado.----------------------------------------------

   // Opciones de la impresin.
   if lPrevio

      if lOrientacion

         SELECT PRINTER cImpresora ;
            TO lSuccess ;
            ORIENTATION PRINTER_ORIENT_LANDSCAPE ;
            PAPERSIZE PRINTER_PAPER_A4 ;
            PREVIEW

      else

         SELECT PRINTER cImpresora ;
            TO lSuccess ;
            ORIENTATION PRINTER_ORIENT_PORTRAIT ;
            PAPERSIZE PRINTER_PAPER_A4 ;
            PREVIEW

      endif

   else

      if lOrientacion

         SELECT PRINTER cImpresora ;
            TO lSuccess ;
            ORIENTATION PRINTER_ORIENT_LANDSCAPE ;
            PAPERSIZE PRINTER_PAPER_A4

      else

         SELECT PRINTER cImpresora ;
            TO lSuccess ;
            ORIENTATION PRINTER_ORIENT_PORTRAIT ;
            PAPERSIZE PRINTER_PAPER_A4

      endif

    endif

   // Control de errores.
   if lSuccess == .F.
      msgExclamation( _HMG_SYSDATA [ 130 ][25], _cTitulo )
      return nil
   endif

   // Inicio del listado.
   lCabecera := .t.
   lSalida   := .t.
   nFila     := 13
   nPagina   := 1

   START PRINTDOC

      START PRINTPAGE

         do while lSalida

            // Cabecera el listado.
            if lCabecera

               @ 5*RF, (HBPRNMAXCOL-HMG_LEN(_cTitulo)-6)*CF PRINT _cTitulo  FONT "COURIER NEW" SIZE 12 BOLD
               @ (6*RF) + 1 , 10*CF PRINT LINE TO (6*RF) + 1 , (HBPRNMAXCOL-5)*CF PENWIDTH 0.2

               @ 7*RF   , 11*CF PRINT _HMG_SYSDATA [ 130 ] [26]   FONT "COURIER NEW" SIZE 9 BOLD
               @ 8*RF   , 11*CF PRINT _HMG_SYSDATA [ 130 ][27] FONT "COURIER NEW" SIZE 9 BOLD
               @ 9*RF   , 11*CF PRINT _HMG_SYSDATA [ 130 ][28] FONT "COURIER NEW" SIZE 9 BOLD
               @ 10*RF  , 11*CF PRINT _HMG_SYSDATA [ 130 ][33] FONT "COURIER NEW" SIZE 9 BOLD

               @ 7*RF   , 23*CF PRINT (_cArea)->( ordName() )  FONT "COURIER NEW" SIZE 9
               @ 8*RF   , 23*CF PRINT cRegistro1      FONT "COURIER NEW" SIZE 9
               @ 9*RF   , 23*CF PRINT cRegistro2      FONT "COURIER NEW" SIZE 9
               @ 10*RF  , 23*CF PRINT _cFiltro        FONT "COURIER NEW" SIZE 9

               nColumna := 10

               for i := 1 to HMG_LEN( aCampo )
                  @ 12*RF, nColumna*CF PRINT RECTANGLE TO 12*RF, nColumna + aAncho[i] PENWIDTH 0.2
                  @ 12*RF, (nColumna + 1) *CF PRINT aCampo[i] FONT "COURIER NEW" SIZE 9 BOLD
                  nColumna += aAncho[i]
               next

               lCabecera := .f.

            endif

            // Registros.
            nColumna := 10
            for i := 1 to HMG_LEN( aNumeroCampo )
               nCampo := aNumeroCampo[i]
               do case
                  case _aEstructura[nCampo,DBS_TYPE] == "N"

                     @ nFila*RF, ( nColumna + aAncho[i] ) *CF PRINT (_cArea)->( FieldGet( aNumeroCampo[i] ) ) FONT "COURIER NEW" SIZE 8

                  case _aEstructura[nCampo,DBS_TYPE] == "L"

                     @ nFila*RF, ( nColumna + 1 )  *CF PRINT iif( (_cArea)->( FieldGet( aNumeroCampo[i] ) ), _HMG_SYSDATA [ 130 ][29], _HMG_SYSDATA [ 130 ][30] ) FONT "COURIER NEW" SIZE 8

                  case _aEstructura[nCampo,DBS_TYPE] == "M"

                     @ nFila*RF, ( nColumna + 1 )  *CF PRINT HB_USUBSTR( (_cArea)->( FieldGet( aNumeroCampo[i] ) ), 1, 20 ) FONT "COURIER NEW" SIZE 8

                  otherwise

                     @ nFila*RF, ( nColumna + 1 )  *CF PRINT (_cArea)->( FieldGet( aNumeroCampo[i] ) ) FONT "COURIER NEW" SIZE 8

               endcase

               nColumna += aAncho[i]

            next

            nFila++

            // Comprueba el final del registro.
            if (_cArea)->( RecNo() ) == nUltimo
               lSalida := .f.
            endif
            if (_cArea)->( EOF())
               lSalida := .f.
            endif
            (_cArea)->( dbSkip( 1 ) )

            // Pie.
            if lOrientacion
               if nFila > 44

                  @ (46*RF) - 1 , 10 *CF  PRINT LINE TO (46*RF)-1, ( HBPRNMAXCOL - 5 ) *CF PENWIDTH 0.2

                  cPie := HB_ValToStr( Date() ) + " " + Time()

                  @ 46*RF, 10 *CF  PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD


                  cPie := "Pagina:" + " " +          ;
                     ALLTRIM( STR( nPagina) ) +      ;
                     "/" +                           ;
                     ALLTRIM( STR( nPaginas ) )

                  @ 46*RF, ( HBPRNMAXCOL - HMG_LEN(cPie)-5 )  *CF PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

                  nPagina++
                  nFila := 13
                  lCabecera := .t.

                  END PRINTPAGE

                  START PRINTPAGE

               endif
            else
               if nFila > 66

                  @ (68*RF)-1, 10 *CF  PRINT LINE TO (68*RF)-1, ( HBPRNMAXCOL- 5 )  *CF PENWIDTH 0.2

                  cPie := HB_ValToStr( Date() ) + " " + Time()

                  @ 68*RF, 10 *CF PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

                  cPie := "Pagina: " +                    ;
                     ALLTRIM( STR( nPagina) ) +      ;
                     "/" +                           ;
                     ALLTRIM( STR( nPaginas ) )

                  @ 68*RF, ( HBPRNMAXCOL - HMG_LEN(cPie)-5 )  *CF PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

                  nFila := 13
                  nPagina++
                  lCabecera := .t.

                  END PRINTPAGE

                  START PRINTPAGE

               endif

            endif
         enddo

      // Comprueba que se imprime el pie de la ultima hoja.----------
      if lOrientacion

         @ (46*RF)-1, 10 *CF  PRINT LINE TO (46*RF)-1, ( HBPRNMAXCOL - 5 )  *CF PENWIDTH 0.2

         cPie := HB_ValToStr( Date() ) + " " + Time()
         @ 46*RF, 10 *CF  PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

         cPie := "Pgina: " +                    ;
            ALLTRIM( STR( nPagina) ) +      ;
            "/" +                           ;
            ALLTRIM( STR( nPaginas ) )
         @ 46*RF, ( HBPRNMAXCOL -HMG_LEN(cPie)-5 )  *CF PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD
      else

         @ (68*RF)-1, 10  *CF PRINT LINE TO (68*RF)-1, ( HBPRNMAXCOL - 5 ) *CF PENWIDTH 0.2
         cPie := HB_ValToStr( Date() ) + " " + Time()
         @ 68*RF, 10 *CF  PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

         cPie := "Pgina: " +                    ;
            ALLTRIM( STR( nPagina) ) +      ;
            "/" +                           ;
            ALLTRIM( STR( nPaginas ) )
         @ 68*RF, ( HBPRNMAXCOL - HMG_LEN(cPie)-5)  *CF PRINT cPie FONT "COURIER NEW" SIZE 9 BOLD

      endif


      END PRINTPAGE

   END PRINTDOC

////////// Cierra la ventana.--------------------------------------------------
   (_cArea)->( dbGoTo( nRegistro ) )
   wndABM3Listado.Release

return NIL        // END MAIN
